@@ -107,6 +107,7 @@ func nullResponse(requestType int, name, pattern string, requestInfo, request []
 func API(threadIndex uint32, state interface{}) (*Instance, error) {
 	protocol := os.Getenv("CLOUDI_API_INIT_PROTOCOL")
 	if protocol == "" {
+		os.Stderr.WriteString("CloudI service execution must occur in CloudI\n")
 		return nil, invalidInputErrorNew()
 	}
 	bufferSize, err := uintGetenv("CLOUDI_API_INIT_BUFFER_SIZE")
@@ -731,6 +732,10 @@ func (api *Instance) callback(command uint32, name, pattern string, requestInfo,
 			case *ForwardSyncError:
 				api.terminate = true
 				return err
+			case *AssertionError:
+				ErrorExit(os.Stderr, err)
+			case *FatalError:
+				ErrorExit(os.Stderr, err)
 			default:
 				os.Stderr.WriteString(err.Error() + "\n")
 				err = nil
@@ -756,6 +761,10 @@ func (api *Instance) callback(command uint32, name, pattern string, requestInfo,
 				return err
 			case *ForwardSyncError:
 				return nil
+			case *AssertionError:
+				ErrorExit(os.Stderr, err)
+			case *FatalError:
+				ErrorExit(os.Stderr, err)
 			default:
 				os.Stderr.WriteString(err.Error() + "\n")
 				err = nil
@@ -785,6 +794,10 @@ func (api *Instance) callbackExecute(function Callback, command uint32, name, pa
 				err = errValue
 			case *ForwardSyncError:
 				err = errValue
+			case *AssertionError:
+				err = errValue
+			case *FatalError:
+				err = errValue
 			default:
 				err = StackErrorWrapNew(errValue)
 			}
@@ -1573,6 +1586,44 @@ func (e *TerminateError) Timeout() uint32 {
 	return e.timeout
 }
 
+// AssertionError indicates an unrecoverable condition in a CloudI service
+type AssertionError struct {
+	stack []byte
+}
+
+// Assert that a value is true
+func Assert(value bool) {
+	if !value {
+		panic(&AssertionError{stack: debug.Stack()})
+	}
+}
+func (e *AssertionError) Error() string {
+	return "Assertion failed !"
+}
+
+// Stack return the stack stored when the error was created
+func (e *AssertionError) Stack() []byte {
+	return e.stack
+}
+
+// FatalError indicates an unrecoverable error in a CloudI service
+type FatalError struct {
+	stack []byte
+}
+
+// Create a FatalError
+func FatalErrorNew() error {
+	return &FatalError{stack: debug.Stack()}
+}
+func (e *FatalError) Error() string {
+	return "Fatal Error"
+}
+
+// Stack return the stack stored when the error was created
+func (e *FatalError) Stack() []byte {
+	return e.stack
+}
+
 // ErrorWrite outputs error information to the cloudi.log file through stderr
 func ErrorWrite(stream *os.File, err error) {
 	output := new(bytes.Buffer)
