@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2018 Michael Truog <mjtruog at gmail dot com>
+// Copyright (c) 2014-2020 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -36,6 +36,14 @@ var fs = require('fs');
 var assert = require('assert');
 
 var toNativeString = {}.toString;
+CloudI.nodejs_version_after = Erlang.nodejs_version_after;
+var bufferFrom;
+if (Erlang.nodejs_version_after('5.10.0',true)) {
+    bufferFrom = Buffer.from;
+}
+else {
+    bufferFrom = Buffer;
+}
 var littleEndian = (
     (new Uint16Array((new Uint8Array([0,1])).buffer))[0] === 0x0100);
 var unpackUint8 = function unpackUint8 (i, data) {
@@ -67,11 +75,33 @@ else {
     unpackUint32 = unpackUint32big;
 }
 var packUint32big = function packUint32big (value) {
-    return new Buffer([(value >>> 24) & 0xff,
-                       (value >>> 16) & 0xff,
-                       (value >>> 8) & 0xff,
-                       value & 0xff]);
+    return new bufferFrom([(value >>> 24) & 0xff,
+                           (value >>> 16) & 0xff,
+                           (value >>> 8) & 0xff,
+                           value & 0xff]);
 };
+if (Erlang.nodejs_version_after('10.0.0',true)) {
+    var originalEmitWarning = process.emitWarning;
+    process.emitWarning = function(warning, type, code, ctor) {
+        if (code === 'DEP0097') {
+            // Ignore the stderr output of the runtime deprecation:
+            // "Using a domain property in MakeCallback is deprecated.
+            //  Use the async_context variant of MakeCallback or the
+            //  AsyncResource class instead."
+            //
+            // Until the exception handling of the domain module is
+            // provided by a different module (like async_hooks) or
+            // exceptions (throw/try/catch) are removed from Javascript,
+            // the use of the domain module needs to remain
+            // (to remain consistent with other CloudI API implementations).
+            // The stderr runtime deprecation information is ignored
+            // because it provides no useful information and is only an
+            // annoyance as a false negative.
+            return;
+        }
+        originalEmitWarning(warning, type, code, ctor);
+    };
+}
 
 var InvalidInputException = function InvalidInputException () {
     var error = new Error('Invalid Input');
@@ -154,6 +184,17 @@ TerminateException.prototype = Object.create(Error.prototype, {
 TerminateException.prototype.timeout = function () {
     return this._timeout;
 };
+var FatalError = function FatalError (message) {
+    var error = new Error(message);
+    error.name = 'FatalError';
+    this.message = error.message;
+    if (error.stack) {
+        this.stack = error.stack;
+    }
+}
+FatalError.prototype = Object.create(Error.prototype, {
+    name: { value: 'FatalError' }
+});
 
 CloudI.stdout_write = function stdout_write (s) {
     fs.writeSync(1, s);
@@ -185,6 +226,7 @@ CloudI.API = function API (thread_index, callback) {
                callback : function (API_) {};
     var protocol_str = process.env.CLOUDI_API_INIT_PROTOCOL;
     if (protocol_str === undefined) {
+        CloudI.stderr_write('CloudI service execution must occur in CloudI\n');
         throw new InvalidInputException();
     }
     var buffer_size_str = process.env.CLOUDI_API_INIT_BUFFER_SIZE;
@@ -203,29 +245,20 @@ CloudI.API = function API (thread_index, callback) {
     else {
         throw new InvalidInputException();
     }
-    if (process.versions['node'] > '0.12.1') {
-        API._s_in = new net.Socket({fd: (thread_index + 3),
-                                    readable: true,
-                                    writable: true});
-        API._s_out = API._s_in;
-    }
-    else {
-        API._s_in = new net.Socket({fd: (thread_index + 3),
-                                    readable: true,
-                                    writable: false});
-        API._s_out = fs.createWriteStream(null, {fd: (thread_index + 3)});
-    }
+    API._socket = new net.Socket({fd: (thread_index + 3),
+                                  readable: true,
+                                  writable: true});
     API._initialization_complete = false;
     API._terminate = false;
     API._terminate_callback = undefined;
     API._size = parseInt(buffer_size_str);
     API._callbacks = {};
-    API._timeout_terminate = 1000; // TIMEOUT_TERMINATE_MIN
+    API._timeout_terminate = 10; // TIMEOUT_TERMINATE_MIN
     API._poll_callback = callback;
     API._poll_callbacks_pending = [];
     API._poll_data = undefined;
     API._poll_data_size = undefined;
-    API._s_in.on('data', function(data) {
+    API._socket.on('data', function(data) {
         try {
             if (! API._use_header) {
                 API._poll_request(data);
@@ -259,7 +292,6 @@ CloudI.API = function API (thread_index, callback) {
                 err instanceof MessageDecodingException) {
                 CloudI.stderr_write(err.stack + '\n');
                 API._poll_terminate();
-                process.exit(1);
             }
             else if (err instanceof TerminateException) {
                 API._poll_terminate();
@@ -267,14 +299,12 @@ CloudI.API = function API (thread_index, callback) {
             else {
                 API._exception(err);
                 API._poll_terminate();
-                process.exit(1);
             }
         }
     });
-    API._s_in.on('error', function(err) {
+    API._socket.on('error', function(err) {
         API._exception(err);
         API._poll_terminate();
-        process.exit(1);
     });
     API._send(new Erlang.OtpErlangAtom('init'));
 };
@@ -557,33 +587,38 @@ CloudI.API.prototype._callback = function (command, name, pattern,
             domain_async.name = 'CloudI Async';
             domain_async.on('error', function(err) {
                 domain_async.exit();
-                if (err instanceof InvalidInputException ||
-                    err instanceof MessageDecodingException) {
-                    CloudI.stderr_write(err.stack + '\n');
-                    API._poll_terminate();
-                    process.exit(1);
+                var return_null_response = false;
+                if (err instanceof MessageDecodingException) {
+                    API._terminate = true;
+                    return_null_response = true;
                 }
                 else if (err instanceof TerminateException) {
-                    API._poll_terminate();
+                    return_null_response = true;
                 }
                 else if (err instanceof ReturnAsyncException) {
                     return;
                 }
                 else if (err instanceof ReturnSyncException) {
+                    API._terminate = true;
                     CloudI.stderr_write(err.stack + '\n');
-                    API._poll_terminate();
-                    process.exit(1);
                 }
                 else if (err instanceof ForwardAsyncException) {
                     return;
                 }
                 else if (err instanceof ForwardSyncException) {
+                    API._terminate = true;
+                    CloudI.stderr_write(err.stack + '\n');
+                }
+                else if (err instanceof assert.AssertionError ||
+                         err instanceof FatalError) {
                     CloudI.stderr_write(err.stack + '\n');
-                    API._poll_terminate();
                     process.exit(1);
                 }
                 else {
+                    return_null_response = true;
                     API._exception(err);
+                }
+                if (return_null_response) {
                     var response_info = '';
                     var response = '';
                     try {
@@ -595,6 +630,9 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                         err_new = undefined;
                     }
                 }
+                if (API._terminate) {
+                    API._poll_terminate();
+                }
             });
             domain_async.enter();
             process.nextTick(function () {
@@ -636,33 +674,38 @@ CloudI.API.prototype._callback = function (command, name, pattern,
             domain_sync.name = 'CloudI Sync';
             domain_sync.on('error', function(err) {
                 domain_sync.exit();
-                if (err instanceof InvalidInputException ||
-                    err instanceof MessageDecodingException) {
-                    CloudI.stderr_write(err.stack + '\n');
-                    API._poll_terminate();
-                    process.exit(1);
+                var return_null_response = false;
+                if (err instanceof MessageDecodingException) {
+                    API._terminate = true;
+                    return_null_response = true;
                 }
                 else if (err instanceof TerminateException) {
-                    API._poll_terminate();
+                    return_null_response = true;
                 }
                 else if (err instanceof ReturnAsyncException) {
+                    API._terminate = true;
                     CloudI.stderr_write(err.stack + '\n');
-                    API._poll_terminate();
-                    process.exit(1);
                 }
                 else if (err instanceof ReturnSyncException) {
                     return;
                 }
                 else if (err instanceof ForwardAsyncException) {
+                    API._terminate = true;
                     CloudI.stderr_write(err.stack + '\n');
-                    API._poll_terminate();
-                    process.exit(1);
                 }
                 else if (err instanceof ForwardSyncException) {
                     return;
                 }
+                else if (err instanceof assert.AssertionError ||
+                         err instanceof FatalError) {
+                    CloudI.stderr_write(err.stack + '\n');
+                    process.exit(1);
+                }
                 else {
+                    return_null_response = true;
                     API._exception(err);
+                }
+                if (return_null_response) {
                     var response_info = '';
                     var response = '';
                     try {
@@ -674,6 +717,9 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                         err_new = undefined;
                     }
                 }
+                if (API._terminate) {
+                    API._poll_terminate();
+                }
             });
             domain_sync.enter();
             process.nextTick(function () {
@@ -727,6 +773,7 @@ CloudI.API.prototype._handle_events = function (data, data_size, i, command) {
     while (true) {
         switch (command) {
             case MESSAGE_TERM:
+                API._terminate = true;
                 throw new TerminateException(API._timeout_terminate);
             case MESSAGE_REINIT:
                 API._process_count = unpackUint32(i, data);
@@ -837,7 +884,6 @@ CloudI.API.prototype._poll_request = function (data) {
                     if (err) {
                         API._exception(err);
                         API._poll_terminate();
-                        process.exit(1);
                     }
                     else {
                         API._callback(command, name, pattern,
@@ -979,9 +1025,11 @@ CloudI.API.prototype._poll_wait = function (f) {
 CloudI.API.prototype._poll_terminate = function () {
     var API = this;
     API._terminate = true;
+    API._socket.destroy();
     if (API._terminate_callback !== undefined) {
         API._terminate_callback(false);
     }
+    process.exit();
 };
 
 CloudI.API.prototype._poll_blocked = function () {
@@ -1042,29 +1090,69 @@ CloudI.API.prototype.poll = function (callback, timeout) {
     }, 0);
 };
 
-CloudI.API.prototype._text_key_value_parse = function (text) {
-    var result = {};
-    var data = text.toString('binary').split('\x00');
-    for (var i = 0; i < data.length - 1; i += 2) {
-        var key = data[i];
-        var current = result[key];
+CloudI.API.prototype.shutdown = function (callback, reason) {
+    callback = typeof callback !== 'undefined' ?
+               callback : function () {};
+    reason = typeof reason !== 'undefined' ?
+             reason : "";
+    this._poll_wait(function (API) {
+        API._send([new Erlang.OtpErlangAtom('shutdown'), reason]);
+        callback();
+    });
+};
+
+// class method
+var text_pairs_parse = function text_pairs_parse (text) {
+    var pairs = {};
+    var text_segments = text.toString('binary').split('\x00');
+    for (var i = 0; i < text_segments.length - 1; i += 2) {
+        var key = text_segments[i];
+        var current = pairs[key];
         if (current === undefined) {
-            result[key] = data[i + 1];
+            pairs[key] = text_segments[i + 1];
         }
         else if (typeof current === 'object' && 
                  toNativeString.call(response) == '[object Array]') {
-            current.push(data[i + 1]);
+            current.push(text_segments[i + 1]);
         }
         else {
-            result[key] = [current, data[i + 1]];
+            pairs[key] = [current, text_segments[i + 1]];
         }
     }
-    return result;
+    return pairs;
 };
 
-CloudI.API.prototype.info_key_value_parse = function (message_info) {
-    var API = this;
-    return API._text_key_value_parse(message_info);
+// class method
+var text_pairs_new = function text_pairs_new (pairs, response) {
+    var text_segments = [];
+    for (var key in pairs) {
+        var values = pairs[key];
+        if (typeof values === 'string') {
+            text_segments.push(key);
+            text_segments.push(values);
+        }
+        else {
+            for (var value in values) {
+                text_segments.push(key);
+                text_segments.push(value);
+            }
+        }
+    }
+    if (response && text_segments.length == 0) {
+        return '\x00';
+    }
+    text_segments.push('');
+    return text_segments.join('\x00');
+};
+
+CloudI.API.info_key_value_parse = function info_key_value_parse (info) {
+    return text_pairs_parse(info);
+};
+
+CloudI.API.info_key_value_new = function info_key_value_new (pairs, response) {
+    response = typeof response !== 'undefined' ?
+               response : true;
+    return text_pairs_new(pairs, response);
 };
 
 CloudI.API.prototype._exception = function (err) {
@@ -1080,13 +1168,11 @@ CloudI.API.prototype._send = function (terms) {
         if (err) {
             API._exception(err);
             API._poll_terminate();
-            process.exit(1);
-            return;
         }
         if (API._use_header) {
             data = Buffer.concat([packUint32big(data.length), data]);
         }
-        API._s_out.write(data, 'binary');
+        API._socket.write(data, 'binary');
     });
 };
 
@@ -1097,5 +1183,6 @@ CloudI.ForwardSyncException = ForwardSyncException;
 CloudI.ForwardAsyncException = ForwardAsyncException;
 CloudI.MessageDecodingException = MessageDecodingException;
 CloudI.TerminateException = TerminateException;
+CloudI.FatalError = FatalError;
 
 };
