@@ -189,6 +189,7 @@ exception ReturnAsync
 exception ForwardSync
 exception ForwardAsync
 exception Terminate
+exception FatalError
 
 let print_exception str =
   prerr_endline ("Exception: " ^ str)
@@ -655,7 +656,13 @@ and callback
           None
         | e ->
           print_exception (backtrace e) ;
-          Some (Null))
+          match e with
+          | Assert_failure _ ->
+            exit 1 ;
+          | FatalError ->
+            exit 1 ;
+          | _ ->
+            Some (Null))
     | SYNC -> (
       try Some (
         callback_f
@@ -678,7 +685,13 @@ and callback
           None
         | e ->
           print_exception (backtrace e) ;
-          Some (Null))
+          match e with
+          | Assert_failure _ ->
+            exit 1 ;
+          | FatalError ->
+            exit 1 ;
+          | _ ->
+            Some (Null))
   in
   let callback_result_type =
     match callback_result_value with
@@ -1050,8 +1063,9 @@ let api
   ?terminate_return_value:(terminate_return_value = true)
   (thread_index : int) (state : 's): ('s Instance.t, string) result =
   let protocol = getenv "CLOUDI_API_INIT_PROTOCOL" in
-  if protocol = "" then
-    Error (invalid_input_error)
+  if protocol = "" then (
+    prerr_endline "CloudI service execution must occur in CloudI" ;
+    Error (invalid_input_error))
   else
     match getenv_to_uint "CLOUDI_API_INIT_BUFFER_SIZE" with
       | Error (error) ->
