@@ -5,7 +5,7 @@
 
   MIT License
 
-  Copyright (c) 2017 Michael Truog <mjtruog at gmail dot com>
+  Copyright (c) 2017-2020 Michael Truog <mjtruog at protonmail dot com>
 
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
@@ -27,9 +27,9 @@
 
  *)
 
-(** OCaml {{:http://cloudi.org/api.html#1_Intro}CloudI API}.
+(** OCaml {{:https://cloudi.org/api.html#1_Intro}CloudI API}.
     Example usage is available in the
-    {{:http://cloudi.org/tutorials.html#cloudi_examples}integration tests}.
+    {{:https://cloudi.org/tutorials.html#cloudi_examples}integration tests}.
  *)
 
 (** provided when handling a service request *)
@@ -76,9 +76,13 @@ exception ReturnSync
 exception ReturnAsync
 exception ForwardSync
 exception ForwardAsync
+exception Terminate
+exception FatalError
 
 (** creates an instance of the CloudI API *)
-val api : int -> 's -> ('s Instance.t, string) result
+val api :
+  ?terminate_return_value:bool ->
+  int -> 's -> ('s Instance.t, string) result
 
 (** returns the thread count from the service configuration *)
 val thread_count : unit -> (int, string) result
@@ -185,6 +189,13 @@ val timeout_terminate : 's Instance.t -> int
 (** blocks to process incoming CloudI service requests *)
 val poll : 's Instance.t -> int -> (bool, string) result
 
-(** parses "text_pairs" in service request info *)
+(** shutdown the service successfully *)
+val shutdown : ?reason:string -> 's Instance.t -> (unit, string) result
+
+(** decode service request info key/value data *)
 val info_key_value_parse : string -> (string, string list) Hashtbl.t
 
+(** encode service response info key/value data *)
+val info_key_value_new :
+  ?response:bool -> (string, string list) Hashtbl.t -> string
+
