@@ -30,7 +30,16 @@ namespace CloudI;
 for ($i = 0; $i < ob_get_level(); $i++) { ob_end_flush(); }
 ob_implicit_flush(true);
 // an assert failure needs to cause php to exit
-assert_options(ASSERT_BAIL);
+if (! ini_get('assert.active'))
+    ini_set('assert.active', '1');
+if (! ini_get('assert.warning'))
+    ini_set('assert.warning', '1');
+if (! ini_get('assert.bail'))
+    ini_set('assert.bail', '1');
+if (ini_get('assert.quiet_eval'))
+    ini_set('assert.quiet_eval', '0');
+if (ini_get('assert.callback'))
+    ini_set('assert.callback', null);
 
 require dirname(__FILE__) . '/Erlang.php';
 
@@ -72,7 +81,10 @@ class API
     {
         $protocol_str = getenv('CLOUDI_API_INIT_PROTOCOL');
         if ($protocol_str == false)
+        {
+            fwrite(STDERR, "CloudI service execution must occur in CloudI\n");
             throw new InvalidInputException();
+        }
         $buffer_size_str = getenv('CLOUDI_API_INIT_BUFFER_SIZE');
         if ($buffer_size_str == false)
             throw new InvalidInputException();
@@ -393,7 +405,7 @@ class API
                 catch (ReturnSyncException $e)
                 {
                     $this->terminate = true;
-                    echo "{$e->getMessage()}\n{$e}\n";
+                    fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
                     return;
                 }
                 catch (ForwardAsyncException $e)
@@ -403,13 +415,18 @@ class API
                 catch (ForwardSyncException $e)
                 {
                     $this->terminate = true;
-                    echo "{$e->getMessage()}\n{$e}\n";
+                    fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
                     return;
                 }
-                catch (\Exception $e)
+                catch (\Error $e)
+                {
+                    fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
+                    exit(1);
+                }
+                catch (\Throwable $e)
                 {
                     $return_null_response = true;
-                    echo "{$e->getMessage()}\n{$e}\n";
+                    fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
                 }
                 if ($return_null_response)
                 {
@@ -465,7 +482,7 @@ class API
                 catch (ReturnAsyncException $e)
                 {
                     $this->terminate = true;
-                    echo "{$e->getMessage()}\n{$e}\n";
+                    fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
                     return;
                 }
                 catch (ForwardSyncException $e)
@@ -475,13 +492,18 @@ class API
                 catch (ForwardAsyncException $e)
                 {
                     $this->terminate = true;
-                    echo "{$e->getMessage()}\n{$e}\n";
+                    fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
                     return;
                 }
-                catch (\Exception $e)
+                catch (\Error $e)
+                {
+                    fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
+                    exit(1);
+                }
+                catch (\Throwable $e)
                 {
                     $return_null_response = true;
-                    echo "{$e->getMessage()}\n{$e}\n";
+                    fwrite(STDERR, "{$e->getMessage()}\n{$e}\n");
                 }
                 if ($return_null_response)
                 {
@@ -1018,4 +1040,12 @@ class TerminateException extends \Exception
     }
 }
 
+class FatalError extends \Error
+{
+    public function __construct($message)
+    {
+        parent::__construct($message, 0, null);
+    }
+}
+
 ?>
