@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2011-2018 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -45,18 +45,21 @@ from erlang import (binary_to_term, term_to_binary,
 
 if sys.version_info[0] >= 3:
     TypeUnicode = str
+    def _function_argc(function):
+        args, _, _, _, _, _, _ = inspect.getfullargspec(function)
+        return len(args)
 else:
     TypeUnicode = unicode
+    def _function_argc(function):
+        # pylint: disable=deprecated-method
+        args, _, _, _ = inspect.getargspec(function)
+        return len(args)
 
 __all__ = [
     'API',
     'InvalidInputException',
     'MessageDecodingException',
     'TerminateException',
-    # XXX backwards-compatibility
-    'invalid_input_exception',
-    'message_decoding_exception',
-    'terminate_exception',
 ]
 
 _MESSAGE_INIT = 1
@@ -73,6 +76,7 @@ _MESSAGE_TERM = 11
 
 # pylint: disable=too-many-instance-attributes
 # pylint: disable=too-many-public-methods
+# pylint: disable=useless-object-inheritance
 class API(object):
     """
     CloudI API object for use in a single thread of execution
@@ -84,6 +88,7 @@ class API(object):
     def __init__(self, thread_index):
         protocol_str = os.getenv('CLOUDI_API_INIT_PROTOCOL')
         if protocol_str is None:
+            sys.stderr.write('CloudI service execution must occur in CloudI\n')
             raise InvalidInputException()
         buffer_size_str = os.getenv('CLOUDI_API_INIT_BUFFER_SIZE')
         if buffer_size_str is None:
@@ -109,7 +114,7 @@ class API(object):
         self.__terminate = False
         self.__size = int(buffer_size_str)
         self.__callbacks = {}
-        self.__timeout_terminate = 1000 # TIMEOUT_TERMINATE_MIN
+        self.__timeout_terminate = 10 # TIMEOUT_TERMINATE_MIN
         self.__send(term_to_binary(OtpErlangAtom(b'init')))
         (self.__process_index,
          self.__process_count,
@@ -135,8 +140,7 @@ class API(object):
         """
         subscribes to a service name pattern with a callback
         """
-        args, _, _, _ = inspect.getargspec(function)
-        if len(args) != 10:
+        if _function_argc(function) != 10:
             # self + arguments for a member function
             #  api + arguments for a static function
             raise InvalidInputException()
@@ -167,7 +171,7 @@ class API(object):
         value = self.__callbacks.get(key, None)
         assert value is not None
         value.popleft()
-        if len(value) == 0:
+        if value == collections.deque([]):
             del self.__callbacks[key]
         self.__send(term_to_binary((OtpErlangAtom(b'unsubscribe'),
                                     pattern)))
@@ -394,12 +398,15 @@ class API(object):
         # pylint: disable=too-many-return-statements
         # pylint: disable=too-many-branches
         # pylint: disable=too-many-statements
+        # pylint: disable=too-many-locals
+        # pylint: disable=broad-except
         function_queue = self.__callbacks.get(pattern, None)
         if function_queue is None:
             function = self.__null_response
         else:
             function = function_queue.popleft()
             function_queue.append(function)
+        return_null_response = False
         if command == _MESSAGE_SEND_ASYNC:
             try:
                 response = function(API.ASYNC, name, pattern,
@@ -407,34 +414,42 @@ class API(object):
                                     timeout, priority, trans_id, pid)
                 if isinstance(response, tuple):
                     response_info, response = response
-                    if not (isinstance(response_info, bytes) or
-                            isinstance(response_info, TypeUnicode)):
+                    if not isinstance(response_info, (bytes, TypeUnicode)):
                         response_info = b''
                 else:
                     response_info = b''
-                if not (isinstance(response, bytes) or
-                        isinstance(response_info, TypeUnicode)):
+                if not isinstance(response, (bytes, TypeUnicode)):
                     response = b''
-            except InvalidInputException as exception:
-                raise exception
-            except MessageDecodingException as exception:
-                raise exception
-            except TerminateException as exception:
-                raise exception
+            except MessageDecodingException:
+                self.__terminate = True
+                return_null_response = True
+            except TerminateException:
+                return_null_response = True
             except ReturnAsyncException:
                 return
             except ReturnSyncException:
+                self.__terminate = True
                 traceback.print_exc(file=sys.stderr)
-                assert False
                 return
             except ForwardAsyncException:
                 return
             except ForwardSyncException:
+                self.__terminate = True
                 traceback.print_exc(file=sys.stderr)
-                assert False
                 return
+            except AssertionError:
+                traceback.print_exc(file=sys.stderr)
+                sys.exit(1)
+            except SystemExit:
+                traceback.print_exc(file=sys.stderr)
+                raise
+            except Exception:
+                return_null_response = True
+                traceback.print_exc(file=sys.stderr)
             except:
                 traceback.print_exc(file=sys.stderr)
+                sys.exit(1)
+            if return_null_response:
                 response_info = b''
                 response = b''
             try:
@@ -444,41 +459,49 @@ class API(object):
             except ReturnAsyncException:
                 pass
             return
-        elif command == _MESSAGE_SEND_SYNC:
+        if command == _MESSAGE_SEND_SYNC:
             try:
                 response = function(API.SYNC, name, pattern,
                                     request_info, request,
                                     timeout, priority, trans_id, pid)
                 if isinstance(response, tuple):
                     response_info, response = response
-                    if not (isinstance(response_info, bytes) or
-                            isinstance(response_info, TypeUnicode)):
+                    if not isinstance(response_info, (bytes, TypeUnicode)):
                         response_info = b''
                 else:
                     response_info = b''
-                if not (isinstance(response, bytes) or
-                        isinstance(response_info, TypeUnicode)):
+                if not isinstance(response, (bytes, TypeUnicode)):
                     response = b''
-            except InvalidInputException as exception:
-                raise exception
-            except MessageDecodingException as exception:
-                raise exception
-            except TerminateException as exception:
-                raise exception
+            except MessageDecodingException:
+                self.__terminate = True
+                return_null_response = True
+            except TerminateException:
+                return_null_response = True
             except ReturnSyncException:
                 return
             except ReturnAsyncException:
+                self.__terminate = True
                 traceback.print_exc(file=sys.stderr)
-                assert False
                 return
             except ForwardSyncException:
                 return
             except ForwardAsyncException:
+                self.__terminate = True
                 traceback.print_exc(file=sys.stderr)
-                assert False
                 return
+            except AssertionError:
+                traceback.print_exc(file=sys.stderr)
+                sys.exit(1)
+            except SystemExit:
+                traceback.print_exc(file=sys.stderr)
+                raise
+            except Exception:
+                return_null_response = True
+                traceback.print_exc(file=sys.stderr)
             except:
                 traceback.print_exc(file=sys.stderr)
+                sys.exit(1)
+            if return_null_response:
                 response_info = b''
                 response = b''
             try:
@@ -488,8 +511,7 @@ class API(object):
             except ReturnSyncException:
                 pass
             return
-        else:
-            raise MessageDecodingException()
+        raise MessageDecodingException()
 
     def __handle_events(self, external, data, data_size, j, command=None):
         # pylint: disable=too-many-arguments
@@ -503,9 +525,8 @@ class API(object):
                 self.__terminate = True
                 if external:
                     return False
-                else:
-                    raise TerminateException(self.__timeout_terminate)
-            elif command == _MESSAGE_REINIT:
+                raise TerminateException(self.__timeout_terminate)
+            if command == _MESSAGE_REINIT:
                 i, j = j, j + 4 + 4 + 4 + 1
                 (self.__process_count,
                  self.__timeout_async, self.__timeout_sync,
@@ -518,7 +539,7 @@ class API(object):
                 raise MessageDecodingException()
             if j > data_size:
                 raise MessageDecodingException()
-            elif j == data_size:
+            if j == data_size:
                 return True
             i, j = j, j + 4
             command = struct.unpack(b'=I', data[i:j])[0]
@@ -529,8 +550,10 @@ class API(object):
         # pylint: disable=too-many-branches
         # pylint: disable=too-many-statements
         if self.__terminate:
-            return False
-        elif external and not self.__initialization_complete:
+            if external:
+                return False
+            raise TerminateException(self.__timeout_terminate)
+        if external and not self.__initialization_complete:
             self.__send(term_to_binary(OtpErlangAtom(b'polling')))
             self.__initialization_complete = True
 
@@ -544,9 +567,9 @@ class API(object):
             timeout_value = timeout * 0.001
         fd_in, _, fd_except = select.select([self.__s], [], [self.__s],
                                             timeout_value)
-        if len(fd_except) > 0:
+        if fd_except != []:
             return False
-        if len(fd_in) == 0:
+        if fd_in == []:
             return True
 
         data = b''
@@ -577,8 +600,7 @@ class API(object):
                         prefix.decode('utf-8'), timeout_initialize,
                         timeout_sync, timeout_async, timeout_terminate,
                         priority_default)
-            elif (command == _MESSAGE_SEND_ASYNC or
-                  command == _MESSAGE_SEND_SYNC):
+            if command in (_MESSAGE_SEND_ASYNC, _MESSAGE_SEND_SYNC):
                 i, j = j, j + 4
                 name_size = struct.unpack(b'=I', data[i:j])[0]
                 i, j = j, j + name_size + 4
@@ -613,8 +635,9 @@ class API(object):
                                 request_info, request,
                                 request_timeout, priority, trans_id,
                                 binary_to_term(pid))
-            elif (command == _MESSAGE_RECV_ASYNC or
-                  command == _MESSAGE_RETURN_SYNC):
+                if self.__terminate:
+                    return False
+            elif command in (_MESSAGE_RECV_ASYNC, _MESSAGE_RETURN_SYNC):
                 i, j = j, j + 4
                 response_info_size = struct.unpack(b'=I', data[i:j])[0]
                 i, j = j, j + response_info_size + 1 + 4
@@ -699,13 +722,13 @@ class API(object):
             if timeout_value is not None:
                 if timeout == 0:
                     return True
-                elif timeout > 0:
+                if timeout > 0:
                     timeout_value = timeout * 0.001
             fd_in, _, fd_except = select.select([self.__s], [], [self.__s],
                                                 timeout_value)
-            if len(fd_except) > 0:
+            if fd_except != []:
                 return False
-            if len(fd_in) == 0:
+            if fd_in == []:
                 return True
 
             data = self.__recv(data)
@@ -720,26 +743,60 @@ class API(object):
         """
         return self.__poll_request(timeout, True)
 
-    def __text_key_value_parse(self, text):
-        # pylint: disable=no-self-use
-        result = {}
+    def shutdown(self, reason=None):
+        """
+        shutdown the service successfully
+        """
+        if reason is None:
+            reason = b''
+        self.__send(term_to_binary((OtpErlangAtom(b'shutdown'),
+                                    reason)))
+
+    @staticmethod
+    def __text_pairs_parse(text):
+        pairs = {}
         data = text.split(b'\0')
         for i in range(0, len(data) - 1, 2):
             key = data[i]
-            current = result.get(key, None)
+            current = pairs.get(key, None)
             if current is None:
-                result[key] = data[i + 1]
+                pairs[key] = data[i + 1]
             elif isinstance(current, list):
                 current.append(data[i + 1])
             else:
-                result[key] = [current, data[i + 1]]
-        return result
+                pairs[key] = [current, data[i + 1]]
+        return pairs
 
-    def info_key_value_parse(self, message_info):
+    @staticmethod
+    def __text_pairs_new(pairs, response):
+        text_segments = []
+        for key, values in pairs.items():
+            if isinstance(values, bytes):
+                text_segments.append(key)
+                text_segments.append(values)
+            else:
+                assert not isinstance(values, str)
+                for value in values:
+                    text_segments.append(key)
+                    text_segments.append(value)
+        if response and text_segments == []:
+            return b'\0'
+        text_segments.append(b'')
+        return b'\0'.join(text_segments)
+
+    @staticmethod
+    def info_key_value_parse(info):
         """
-        parses "text_pairs" in service request info
+        decode service request info key/value data
         """
-        return self.__text_key_value_parse(message_info)
+        return API.__text_pairs_parse(info)
+
+    @staticmethod
+    def info_key_value_new(pairs, response=True):
+        """
+        encode service response info key/value data
+        """
+        return API.__text_pairs_new(pairs, response)
 
     def __send(self, data):
         if self.__use_header:
@@ -770,7 +827,7 @@ class API(object):
                 ready = (len(fragment) == self.__size)
                 if ready:
                     fd_in, _, _ = select.select([self.__s], [], [], 0)
-                    ready = (len(fd_in) > 0)
+                    ready = (fd_in != [])
         return data
 
 class InvalidInputException(Exception):
@@ -779,8 +836,6 @@ class InvalidInputException(Exception):
     """
     def __init__(self):
         Exception.__init__(self, 'Invalid Input')
-# XXX backwards-compatibility
-invalid_input_exception = InvalidInputException
 
 class ReturnSyncException(Exception):
     """
@@ -816,8 +871,6 @@ class MessageDecodingException(Exception):
     """
     def __init__(self):
         Exception.__init__(self, 'Message Decoding Error')
-# XXX backwards-compatibility
-message_decoding_exception = MessageDecodingException
 
 class TerminateException(Exception):
     """
@@ -832,14 +885,21 @@ class TerminateException(Exception):
         return the termination timeout
         """
         return self.__timeout
-# XXX backwards-compatibility
-terminate_exception = TerminateException
+
+class FatalError(BaseException):
+    """
+    Fatal Error
+    """
+    def __init__(self, message):
+        BaseException.__init__(self, message)
+
 
 # force unbuffered stdout/stderr handling without external configuration
 if sys.stderr.__class__.__name__ != '_unbuffered':
     class _unbuffered(object):
         # pylint: disable=too-few-public-methods
         def __init__(self, stream):
+            # pylint: disable=import-outside-toplevel
             if sys.version_info[0] >= 3:
                 import io
                 self.__stream = io.TextIOWrapper(
@@ -852,7 +912,8 @@ if sys.stderr.__class__.__name__ != '_unbuffered':
                 )
             else:
                 import codecs
-                self.__stream = codecs.getwriter('UTF-8')(stream)
+                self.encoding = 'UTF-8'
+                self.__stream = codecs.getwriter(self.encoding)(stream)
 
         def write(self, data):
             """
@@ -866,4 +927,3 @@ if sys.stderr.__class__.__name__ != '_unbuffered':
 
     sys.stdout = _unbuffered(sys.stdout)
     sys.stderr = _unbuffered(sys.stderr)
-
