@@ -32,6 +32,7 @@ integration tests <https://cloudi.org/tutorials.html#cloudi_examples>.
 
 import sys
 import os
+import traceback
 import inspect
 from functools import partial
 import libcloudi_py
@@ -65,11 +66,10 @@ class API(object):
     SYNC = -1
 
     def __init__(self, thread_index):
-        # pylint: disable=broad-except
-        try:
-            self.__api = libcloudi_py.cloudi_c(thread_index)
-        except Exception as exception:
-            API.__rethrow_exception(exception)
+        self.__api = libcloudi_py.cloudi_c(thread_index,
+                                           InvalidInputException,
+                                           MessageDecodingException,
+                                           TerminateException)
 
     @staticmethod
     def thread_count():
@@ -97,11 +97,7 @@ class API(object):
         """
         returns the number of subscriptions for a single service name pattern
         """
-        # pylint: disable=broad-except
-        try:
-            return self.__api.subscribe_count(pattern)
-        except Exception as exception:
-            API.__rethrow_exception(exception)
+        return self.__api.subscribe_count(pattern)
 
     def unsubscribe(self, pattern):
         """
@@ -114,7 +110,6 @@ class API(object):
         """
         sends an asynchronous service request
         """
-        # pylint: disable=broad-except
         # pylint: disable=too-many-arguments
         kwargs = {}
         if timeout is not None:
@@ -123,17 +118,13 @@ class API(object):
             kwargs['request_info'] = request_info
         if priority is not None:
             kwargs['priority'] = priority
-        try:
-            return self.__api.send_async(name, request, **kwargs)
-        except Exception as exception:
-            API.__rethrow_exception(exception)
+        return self.__api.send_async(name, request, **kwargs)
 
     def send_sync(self, name, request,
                   timeout=None, request_info=None, priority=None):
         """
         sends a synchronous service request
         """
-        # pylint: disable=broad-except
         # pylint: disable=too-many-arguments
         kwargs = {}
         if timeout is not None:
@@ -142,10 +133,7 @@ class API(object):
             kwargs['request_info'] = request_info
         if priority is not None:
             kwargs['priority'] = priority
-        try:
-            return self.__api.send_sync(name, request, **kwargs)
-        except Exception as exception:
-            API.__rethrow_exception(exception)
+        return self.__api.send_sync(name, request, **kwargs)
 
     def mcast_async(self, name, request,
                     timeout=None, request_info=None, priority=None):
@@ -153,7 +141,6 @@ class API(object):
         sends asynchronous service requests to all subscribers
         of the matching service name pattern
         """
-        # pylint: disable=broad-except
         # pylint: disable=too-many-arguments
         kwargs = {}
         if timeout is not None:
@@ -162,11 +149,7 @@ class API(object):
             kwargs['request_info'] = request_info
         if priority is not None:
             kwargs['priority'] = priority
-        exception = None
-        try:
-            trans_ids = self.__api.mcast_async(name, request, **kwargs)
-        except Exception as exception:
-            API.__rethrow_exception(exception)
+        trans_ids = self.__api.mcast_async(name, request, **kwargs)
         if trans_ids is None:
             return tuple()
         return tuple([
@@ -247,7 +230,6 @@ class API(object):
         """
         blocks to receive an asynchronous service request response
         """
-        # pylint: disable=broad-except
         kwargs = {}
         if timeout is not None:
             kwargs['timeout'] = timeout
@@ -255,10 +237,7 @@ class API(object):
             kwargs['trans_id'] = trans_id
         if consume is not None:
             kwargs['consume'] = consume
-        try:
-            return self.__api.recv_async(**kwargs)
-        except Exception as exception:
-            API.__rethrow_exception(exception)
+        return self.__api.recv_async(**kwargs)
 
     def process_index(self):
         """
@@ -318,13 +297,13 @@ class API(object):
         """
         blocks to process incoming CloudI service requests
         """
-        # pylint: disable=broad-except
         if timeout is None:
             timeout = -1
         try:
             return self.__api.poll(timeout)
-        except Exception as exception:
-            API.__rethrow_exception(exception)
+        except AssertionError:
+            traceback.print_exc(file=sys.stderr)
+            sys.exit(1)
 
     def shutdown(self, reason=None):
         """
@@ -335,16 +314,6 @@ class API(object):
             kwargs['reason'] = reason
         return self.__api.shutdown(**kwargs)
 
-    @staticmethod
-    def __rethrow_exception(exception):
-        if isinstance(exception, libcloudi_py.message_decoding_exception):
-            raise MessageDecodingException(str(exception))
-        if isinstance(exception, libcloudi_py.invalid_input_exception):
-            raise InvalidInputException(str(exception))
-        if isinstance(exception, libcloudi_py.terminate_exception):
-            raise TerminateException(exception.timeout)
-        raise exception
-
     @staticmethod
     def __text_pairs_parse(text):
         pairs = {}
@@ -449,6 +418,14 @@ class TerminateException(Exception):
         """
         return self.__timeout
 
+class FatalError(BaseException):
+    """
+    Fatal Error
+    """
+    def __init__(self, message):
+        BaseException.__init__(self, message)
+
+
 # force unbuffered stdout/stderr handling without external configuration
 if sys.stderr.__class__.__name__ != '_unbuffered':
     class _unbuffered(object):
