@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2014-2017 Michael Truog <mjtruog at gmail dot com>
+# Copyright (c) 2014-2019 Michael Truog <mjtruog at protonmail dot com>
 # Copyright (c) 2009-2013 Dmitry Vasiliev <dima@hlabs.org>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
@@ -70,7 +70,7 @@ end
 
 class ImproperListTestCase < Test::Unit::TestCase
     def test_improper_list
-        lst = Erlang::OtpErlangList.new([1, 2, 3, 4], improper=true)
+        lst = Erlang::OtpErlangList.new([1, 2, 3, 4], true)
         assert(lst.kind_of?(Erlang::OtpErlangList))
         assert_equal(Erlang::OtpErlangList.new([1, 2, 3, 4]).value, lst.value)
         assert_equal(4, lst.value[-1])
@@ -89,7 +89,7 @@ class ImproperListTestCase < Test::Unit::TestCase
     end
     def test_errors
         assert_raise(Erlang::OutputException){
-            Erlang::OtpErlangList.new('invalid', improper=true).binary
+            Erlang::OtpErlangList.new('invalid', true).binary
         }
     end
 end
@@ -331,6 +331,67 @@ class DecodeTestCase < Test::Unit::TestCase
         assert_equal(-6618611909121,
                      Erlang::binary_to_term("\x83o\0\0\0\6\1\1\2\3\4\5\6"))
     end
+    def test_binary_to_term_pid
+        pid_old_binary = (
+            "\x83\x67\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F" \
+            "\x68\x6F\x73\x74\x00\x00\x00\x4E\x00\x00\x00\x00\x00"
+        )
+        pid_old = Erlang::binary_to_term(pid_old_binary)
+        assert(pid_old.kind_of?(Erlang::OtpErlangPid))
+        assert_equal(Erlang::term_to_binary(pid_old),
+                     "\x83gs\rnonode@nohost\x00\x00\x00N" \
+                     "\x00\x00\x00\x00\x00")
+        pid_new_binary = (
+            "\x83\x58\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F\x68" \
+            "\x6F\x73\x74\x00\x00\x00\x4E\x00\x00\x00\x00\x00\x00\x00\x00"
+        )
+        pid_new = Erlang::binary_to_term(pid_new_binary)
+        assert(pid_new.kind_of?(Erlang::OtpErlangPid))
+        assert_equal(Erlang::term_to_binary(pid_new),
+                     "\x83Xs\rnonode@nohost\x00\x00\x00N" \
+                     "\x00\x00\x00\x00\x00\x00\x00\x00")
+    end
+    def test_binary_to_term_port
+        port_old_binary = (
+            "\x83\x66\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F\x68" \
+            "\x6F\x73\x74\x00\x00\x00\x06\x00"
+        )
+        port_old = Erlang::binary_to_term(port_old_binary)
+        assert(port_old.kind_of?(Erlang::OtpErlangPort))
+        assert_equal(Erlang::term_to_binary(port_old),
+                     "\x83fs\rnonode@nohost\x00\x00\x00\x06\x00")
+        port_new_binary = (
+            "\x83\x59\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E\x6F\x68" \
+            "\x6F\x73\x74\x00\x00\x00\x06\x00\x00\x00\x00"
+        )
+        port_new = Erlang::binary_to_term(port_new_binary)
+        assert(port_new.kind_of?(Erlang::OtpErlangPort))
+        assert_equal(Erlang::term_to_binary(port_new),
+                     "\x83Ys\rnonode@nohost\x00\x00\x00\x06" \
+                     "\x00\x00\x00\x00")
+    end
+    def test_binary_to_term_ref
+        ref_new_binary = (
+            "\x83\x72\x00\x03\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E" \
+            "\x6F\x68\x6F\x73\x74\x00\x00\x03\xE8\x4E\xE7\x68\x00\x02\xA4" \
+            "\xC8\x53\x40"
+        )
+        ref_new = Erlang::binary_to_term(ref_new_binary)
+        assert(ref_new.kind_of?(Erlang::OtpErlangReference))
+        assert_equal(Erlang::term_to_binary(ref_new),
+                     "\x83r\x00\x03s\rnonode@nohost\x00\x00\x03\xe8" \
+                     "N\xe7h\x00\x02\xa4\xc8S@")
+        ref_newer_binary = (
+            "\x83\x5A\x00\x03\x64\x00\x0D\x6E\x6F\x6E\x6F\x64\x65\x40\x6E" \
+            "\x6F\x68\x6F\x73\x74\x00\x00\x00\x00\x00\x01\xAC\x03\xC7\x00" \
+            "\x00\x04\xBB\xB2\xCA\xEE"
+        )
+        ref_newer = Erlang::binary_to_term(ref_newer_binary)
+        assert(ref_newer.kind_of?(Erlang::OtpErlangReference))
+        assert_equal(Erlang::term_to_binary(ref_newer),
+                     "\x83Z\x00\x03s\rnonode@nohost\x00\x00\x00\x00\x00" \
+                     "\x01\xac\x03\xc7\x00\x00\x04\xbb\xb2\xca\xee")
+    end
     def test_binary_to_term_compressed_term
         assert_raise(Erlang::ParseException){
             Erlang::binary_to_term("\x83P")
@@ -524,9 +585,10 @@ class EncodeTestCase < Test::Unit::TestCase
         assert_equal("\x83k\0\4test",
                      Erlang::term_to_binary('test'))
     end
-    def test_term_to_binary_boolean
+    def test_term_to_binary_predefined_atom
         assert_equal("\x83s\4true", Erlang::term_to_binary(true))
         assert_equal("\x83s\5false", Erlang::term_to_binary(false))
+        assert_equal("\x83s\x09undefined", Erlang::term_to_binary(nil))
     end
     def test_term_to_binary_short_integer
         assert_equal("\x83a\0", Erlang::term_to_binary(0))
