@@ -27,7 +27,7 @@
 -export([init/1, handle_event/2]).
 
 
--spec parser(Handler::module(), State::any(), Config::list()) -> cloudi_x_jsx:parser().
+-spec parser(Handler::module(), State::any(), Config::cloudi_x_jsx_config:options()) -> cloudi_x_jsx:parser().
 
 parser(Handler, State, Config) ->
     fun(Tokens) -> value(Tokens, {Handler, Handler:init(State)}, [], cloudi_x_jsx_config:parse_config(Config)) end.
@@ -38,7 +38,7 @@ parser(Handler, State, Config) ->
 -spec resume(
         Rest::cloudi_x_jsx:token(),
         State::atom(),
-        Handler::{atom(), any()},
+        Handler::module(),
         Stack::list(atom()),
         Config::cloudi_x_jsx:config()
     ) -> cloudi_x_jsx:parser() | {incomplete, cloudi_x_jsx:parser()}.
@@ -630,7 +630,7 @@ to_hex(X) -> X + 48.    %% ascii "1" is [49], "2" is [50], etc...
 
 
 %% for raw input
--spec init(proplists:proplist()) -> list().
+-spec init([]) -> [].
 
 init([]) -> [].
 
