@@ -1,4 +1,7 @@
-%%% Copyright 2010-2016 Manolis Papadakis <manopapad@gmail.com>,
+%%% -*- coding: utf-8 -*-
+%%% -*- erlang-indent-level: 2 -*-
+%%% -------------------------------------------------------------------
+%%% Copyright 2010-2017 Manolis Papadakis <manopapad@gmail.com>,
 %%%                     Eirini Arvaniti <eirinibob@gmail.com>
 %%%                 and Kostis Sagonas <kostis@cs.ntua.gr>
 %%%
@@ -17,7 +20,7 @@
 %%% You should have received a copy of the GNU General Public License
 %%% along with PropEr.  If not, see <http://www.gnu.org/licenses/>.
 
-%%% @copyright 2010-2016 Manolis Papadakis, Eirini Arvaniti and Kostis Sagonas
+%%% @copyright 2010-2017 Manolis Papadakis, Eirini Arvaniti and Kostis Sagonas
 %%% @version {@version}
 %%% @author Manolis Papadakis
 
@@ -210,7 +213,7 @@ pick(RawType, Size, Seed) ->
     proper:global_state_init_size_seed(Size, Seed),
     case clean_instance(safe_generate(RawType)) of
 	{ok,Instance} = Result ->
-	    Msg = "WARNING: Some garbage has been left in the process registry "
+	    Msg = "WARNING: Some garbage has been left in the process dictionary "
 		  "and the code server~n"
 		  "to allow for the returned function(s) to run normally.~n"
 		  "Please run proper:global_state_erase() when done.~n",
@@ -325,23 +328,23 @@ alt_gens(Type) ->
 	error         -> []
     end.
 
+-compile({inline, [clean_instance/1]}).
 %% @private
 -spec clean_instance(imm_instance()) -> instance().
 clean_instance({'$used',_ImmParts,ImmInstance}) ->
     clean_instance(ImmInstance);
 clean_instance({'$to_part',ImmInstance}) ->
     clean_instance(ImmInstance);
-clean_instance(ImmInstance) ->
-    if
-	is_list(ImmInstance) ->
-	    %% CAUTION: this must handle improper lists
-	    proper_arith:safe_map(fun clean_instance/1, ImmInstance);
-	is_tuple(ImmInstance) ->
-	    proper_arith:tuple_map(fun clean_instance/1, ImmInstance);
-	true ->
-	    ImmInstance
-    end.
+clean_instance(ImmInstance) when is_list(ImmInstance) ->
+    clean_instance_list(ImmInstance);
+clean_instance(ImmInstance) when is_tuple(ImmInstance) ->
+    list_to_tuple(clean_instance_list(tuple_to_list(ImmInstance)));
+clean_instance(ImmInstance) -> ImmInstance.
 
+%% CAUTION: this must handle improper lists
+clean_instance_list([H|T]) -> [clean_instance(H) | clean_instance_list(T)];
+clean_instance_list([])    -> [];
+clean_instance_list(Other) -> clean_instance(Other).
 
 %%-----------------------------------------------------------------------------
 %% Basic type generators
