@@ -2,19 +2,21 @@
 
 -module('Elixir.CloudILogger').
 
--compile(no_auto_import).
+-compile([no_auto_import]).
 
 -export(['MACRO-function_arity'/1,
-	 'MACRO-function_name'/1, 'MACRO-log_debug'/3,
-	 'MACRO-log_debug_apply'/3, 'MACRO-log_debug_apply'/4,
-	 'MACRO-log_debug_sync'/3, 'MACRO-log_error'/3,
-	 'MACRO-log_error_apply'/3, 'MACRO-log_error_apply'/4,
-	 'MACRO-log_error_sync'/3, 'MACRO-log_fatal'/3,
-	 'MACRO-log_fatal_apply'/3, 'MACRO-log_fatal_apply'/4,
-	 'MACRO-log_fatal_sync'/3, 'MACRO-log_info'/3,
-	 'MACRO-log_info_apply'/3, 'MACRO-log_info_apply'/4,
-	 'MACRO-log_info_sync'/3, 'MACRO-log_metadata_get'/1,
-	 'MACRO-log_metadata_set'/2, 'MACRO-log_trace'/3,
+	 'MACRO-function_name'/1, 'MACRO-log'/4,
+	 'MACRO-log_apply'/4, 'MACRO-log_apply'/5,
+	 'MACRO-log_debug'/3, 'MACRO-log_debug_apply'/3,
+	 'MACRO-log_debug_apply'/4, 'MACRO-log_debug_sync'/3,
+	 'MACRO-log_error'/3, 'MACRO-log_error_apply'/3,
+	 'MACRO-log_error_apply'/4, 'MACRO-log_error_sync'/3,
+	 'MACRO-log_fatal'/3, 'MACRO-log_fatal_apply'/3,
+	 'MACRO-log_fatal_apply'/4, 'MACRO-log_fatal_sync'/3,
+	 'MACRO-log_info'/3, 'MACRO-log_info_apply'/3,
+	 'MACRO-log_info_apply'/4, 'MACRO-log_info_sync'/3,
+	 'MACRO-log_metadata_get'/1, 'MACRO-log_metadata_set'/2,
+	 'MACRO-log_sync'/4, 'MACRO-log_trace'/3,
 	 'MACRO-log_trace_apply'/3, 'MACRO-log_trace_apply'/4,
 	 'MACRO-log_trace_sync'/3, 'MACRO-log_warn'/3,
 	 'MACRO-log_warn_apply'/3, 'MACRO-log_warn_apply'/4,
@@ -26,33 +28,33 @@
 '__info__'(module) -> 'Elixir.CloudILogger';
 '__info__'(functions) -> [];
 '__info__'(macros) ->
-    [{function_arity, 0}, {function_name, 0},
-     {log_debug, 2}, {log_debug_apply, 2},
-     {log_debug_apply, 3}, {log_debug_sync, 2},
-     {log_error, 2}, {log_error_apply, 2},
-     {log_error_apply, 3}, {log_error_sync, 2},
-     {log_fatal, 2}, {log_fatal_apply, 2},
-     {log_fatal_apply, 3}, {log_fatal_sync, 2},
-     {log_info, 2}, {log_info_apply, 2}, {log_info_apply, 3},
-     {log_info_sync, 2}, {log_metadata_get, 0},
-     {log_metadata_set, 1}, {log_trace, 2},
-     {log_trace_apply, 2}, {log_trace_apply, 3},
-     {log_trace_sync, 2}, {log_warn, 2}, {log_warn_apply, 2},
-     {log_warn_apply, 3}, {log_warn_sync, 2}];
-'__info__'(attributes) ->
-    erlang:get_module_info('Elixir.CloudILogger',
-			   attributes);
-'__info__'(compile) ->
-    erlang:get_module_info('Elixir.CloudILogger', compile);
-'__info__'(md5) ->
-    erlang:get_module_info('Elixir.CloudILogger', md5);
+    [{function_arity, 0}, {function_name, 0}, {log, 3},
+     {log_apply, 3}, {log_apply, 4}, {log_debug, 2},
+     {log_debug_apply, 2}, {log_debug_apply, 3},
+     {log_debug_sync, 2}, {log_error, 2},
+     {log_error_apply, 2}, {log_error_apply, 3},
+     {log_error_sync, 2}, {log_fatal, 2},
+     {log_fatal_apply, 2}, {log_fatal_apply, 3},
+     {log_fatal_sync, 2}, {log_info, 2}, {log_info_apply, 2},
+     {log_info_apply, 3}, {log_info_sync, 2},
+     {log_metadata_get, 0}, {log_metadata_set, 1},
+     {log_sync, 3}, {log_trace, 2}, {log_trace_apply, 2},
+     {log_trace_apply, 3}, {log_trace_sync, 2},
+     {log_warn, 2}, {log_warn_apply, 2}, {log_warn_apply, 3},
+     {log_warn_sync, 2}];
+'__info__'(EKey = attributes) ->
+    erlang:get_module_info('Elixir.CloudILogger', EKey);
+'__info__'(EKey = compile) ->
+    erlang:get_module_info('Elixir.CloudILogger', EKey);
+'__info__'(EKey = md5) ->
+    erlang:get_module_info('Elixir.CloudILogger', EKey);
 '__info__'(deprecated) -> [].
 
 'MACRO-function_arity'(_E@CALLER) ->
     {'case', [],
      [{{'.', [],
 	[{'__ENV__', [], 'Elixir.CloudILogger'}, function]},
-       [], []},
+       [{no_parens, true}], []},
       [{do,
 	[{'->', [], [[nil], undefined]},
 	 {'->', [],
@@ -64,7 +66,7 @@
     {'case', [],
      [{{'.', [],
 	[{'__ENV__', [], 'Elixir.CloudILogger'}, function]},
-       [], []},
+       [{no_parens, true}], []},
       [{do,
 	[{'->', [], [[nil], undefined]},
 	 {'->', [],
@@ -72,7 +74,8 @@
 	     {'_', [], 'Elixir.CloudILogger'}}],
 	   {function, [], 'Elixir.CloudILogger'}]}]}]]}.
 
-'MACRO-log_debug'(_E@CALLER, EVformat@1, EVargs@1) ->
+'MACRO-log'(_E@CALLER, _Elevel@1, _Eformat@1,
+	    _Eargs@1) ->
     {'__block__', [],
      [{'=', [],
        [{{function, [], 'Elixir.CloudILogger'},
@@ -80,7 +83,42 @@
 	{'case', [],
 	 [{{'.', [],
 	    [{'__ENV__', [], 'Elixir.CloudILogger'}, function]},
-	   [], []},
+	   [{no_parens, true}], []},
+	  [{do,
+	    [{'->', [], [[nil], {undefined, undefined}]},
+	     {'->', [],
+	      [[{'=', [],
+		 [{{'_', [], 'Elixir.CloudILogger'},
+		   {'_', [], 'Elixir.CloudILogger'}},
+		  {function_arity, [], 'Elixir.CloudILogger'}]}],
+	       {function_arity, [], 'Elixir.CloudILogger'}]}]}]]}]},
+      {{'.', [], [cloudi_core_i_logger_interface, log]}, [],
+       [{'__MODULE__', [], 'Elixir.CloudILogger'},
+	{{'.', [],
+	  [{'__ENV__', [], 'Elixir.CloudILogger'}, line]},
+	 [{no_parens, true}], []},
+	{function, [], 'Elixir.CloudILogger'},
+	{arity, [], 'Elixir.CloudILogger'}, _Elevel@1,
+	_Eformat@1, _Eargs@1]}]}.
+
+'MACRO-log_apply'(_E@CALLER, _Elevel@1, _Ef@1, _Ea@1) ->
+    {{'.', [], [cloudi_core_i_logger_interface, log_apply]},
+     [], [_Elevel@1, _Ef@1, _Ea@1]}.
+
+'MACRO-log_apply'(_E@CALLER, _Elevel@1, _Em@1, _Ef@1,
+		  _Ea@1) ->
+    {{'.', [], [cloudi_core_i_logger_interface, log_apply]},
+     [], [_Elevel@1, _Em@1, _Ef@1, _Ea@1]}.
+
+'MACRO-log_debug'(_E@CALLER, _Eformat@1, _Eargs@1) ->
+    {'__block__', [],
+     [{'=', [],
+       [{{function, [], 'Elixir.CloudILogger'},
+	 {arity, [], 'Elixir.CloudILogger'}},
+	{'case', [],
+	 [{{'.', [],
+	    [{'__ENV__', [], 'Elixir.CloudILogger'}, function]},
+	   [{no_parens, true}], []},
 	  [{do,
 	    [{'->', [], [[nil], {undefined, undefined}]},
 	     {'->', [],
@@ -93,24 +131,24 @@
        [{'__MODULE__', [], 'Elixir.CloudILogger'},
 	{{'.', [],
 	  [{'__ENV__', [], 'Elixir.CloudILogger'}, line]},
-	 [], []},
+	 [{no_parens, true}], []},
 	{function, [], 'Elixir.CloudILogger'},
-	{arity, [], 'Elixir.CloudILogger'}, EVformat@1,
-	EVargs@1]}]}.
+	{arity, [], 'Elixir.CloudILogger'}, _Eformat@1,
+	_Eargs@1]}]}.
 
-'MACRO-log_debug_apply'(_E@CALLER, EVf@1, EVa@1) ->
+'MACRO-log_debug_apply'(_E@CALLER, _Ef@1, _Ea@1) ->
     {{'.', [],
       [cloudi_core_i_logger_interface, debug_apply]},
-     [], [EVf@1, EVa@1]}.
+     [], [_Ef@1, _Ea@1]}.
 
-'MACRO-log_debug_apply'(_E@CALLER, EVm@1, EVf@1,
-			EVa@1) ->
+'MACRO-log_debug_apply'(_E@CALLER, _Em@1, _Ef@1,
+			_Ea@1) ->
     {{'.', [],
       [cloudi_core_i_logger_interface, debug_apply]},
-     [], [EVm@1, EVf@1, EVa@1]}.
+     [], [_Em@1, _Ef@1, _Ea@1]}.
 
-'MACRO-log_debug_sync'(_E@CALLER, EVformat@1,
-		       EVargs@1) ->
+'MACRO-log_debug_sync'(_E@CALLER, _Eformat@1,
+		       _Eargs@1) ->
     {'__block__', [],
      [{'=', [],
        [{{function, [], 'Elixir.CloudILogger'},
@@ -118,7 +156,7 @@
 	{'case', [],
 	 [{{'.', [],
 	    [{'__ENV__', [], 'Elixir.CloudILogger'}, function]},
-	   [], []},
+	   [{no_parens, true}], []},
 	  [{do,
 	    [{'->', [], [[nil], {undefined, undefined}]},
 	     {'->', [],
@@ -133,12 +171,12 @@
        [{'__MODULE__', [], 'Elixir.CloudILogger'},
 	{{'.', [],
 	  [{'__ENV__', [], 'Elixir.CloudILogger'}, line]},
-	 [], []},
+	 [{no_parens, true}], []},
 	{function, [], 'Elixir.CloudILogger'},
-	{arity, [], 'Elixir.CloudILogger'}, EVformat@1,
-	EVargs@1]}]}.
+	{arity, [], 'Elixir.CloudILogger'}, _Eformat@1,
+	_Eargs@1]}]}.
 
-'MACRO-log_error'(_E@CALLER, EVformat@1, EVargs@1) ->
+'MACRO-log_error'(_E@CALLER, _Eformat@1, _Eargs@1) ->
     {'__block__', [],
      [{'=', [],
        [{{function, [], 'Elixir.CloudILogger'},
@@ -146,7 +184,7 @@
 	{'case', [],
 	 [{{'.', [],
 	    [{'__ENV__', [], 'Elixir.CloudILogger'}, function]},
-	   [], []},
+	   [{no_parens, true}], []},
 	  [{do,
 	    [{'->', [], [[nil], {undefined, undefined}]},
 	     {'->', [],
@@ -159,24 +197,24 @@
        [{'__MODULE__', [], 'Elixir.CloudILogger'},
 	{{'.', [],
 	  [{'__ENV__', [], 'Elixir.CloudILogger'}, line]},
-	 [], []},
+	 [{no_parens, true}], []},
 	{function, [], 'Elixir.CloudILogger'},
-	{arity, [], 'Elixir.CloudILogger'}, EVformat@1,
-	EVargs@1]}]}.
+	{arity, [], 'Elixir.CloudILogger'}, _Eformat@1,
+	_Eargs@1]}]}.
 
-'MACRO-log_error_apply'(_E@CALLER, EVf@1, EVa@1) ->
+'MACRO-log_error_apply'(_E@CALLER, _Ef@1, _Ea@1) ->
     {{'.', [],
       [cloudi_core_i_logger_interface, error_apply]},
-     [], [EVf@1, EVa@1]}.
+     [], [_Ef@1, _Ea@1]}.
 
-'MACRO-log_error_apply'(_E@CALLER, EVm@1, EVf@1,
-			EVa@1) ->
+'MACRO-log_error_apply'(_E@CALLER, _Em@1, _Ef@1,
+			_Ea@1) ->
     {{'.', [],
       [cloudi_core_i_logger_interface, error_apply]},
-     [], [EVm@1, EVf@1, EVa@1]}.
+     [], [_Em@1, _Ef@1, _Ea@1]}.
 
-'MACRO-log_error_sync'(_E@CALLER, EVformat@1,
-		       EVargs@1) ->
+'MACRO-log_error_sync'(_E@CALLER, _Eformat@1,
+		       _Eargs@1) ->
     {'__block__', [],
      [{'=', [],
        [{{function, [], 'Elixir.CloudILogger'},
@@ -184,7 +222,7 @@
 	{'case', [],
 	 [{{'.', [],
 	    [{'__ENV__', [], 'Elixir.CloudILogger'}, function]},
-	   [], []},
+	   [{no_parens, true}], []},
 	  [{do,
 	    [{'->', [], [[nil], {undefined, undefined}]},
 	     {'->', [],
@@ -199,12 +237,12 @@
        [{'__MODULE__', [], 'Elixir.CloudILogger'},
 	{{'.', [],
 	  [{'__ENV__', [], 'Elixir.CloudILogger'}, line]},
-	 [], []},
+	 [{no_parens, true}], []},
 	{function, [], 'Elixir.CloudILogger'},
-	{arity, [], 'Elixir.CloudILogger'}, EVformat@1,
-	EVargs@1]}]}.
+	{arity, [], 'Elixir.CloudILogger'}, _Eformat@1,
+	_Eargs@1]}]}.
 
-'MACRO-log_fatal'(_E@CALLER, EVformat@1, EVargs@1) ->
+'MACRO-log_fatal'(_E@CALLER, _Eformat@1, _Eargs@1) ->
     {'__block__', [],
      [{'=', [],
        [{{function, [], 'Elixir.CloudILogger'},
@@ -212,7 +250,7 @@
 	{'case', [],
 	 [{{'.', [],
 	    [{'__ENV__', [], 'Elixir.CloudILogger'}, function]},
-	   [], []},
+	   [{no_parens, true}], []},
 	  [{do,
 	    [{'->', [], [[nil], {undefined, undefined}]},
 	     {'->', [],
@@ -225,24 +263,24 @@
        [{'__MODULE__', [], 'Elixir.CloudILogger'},
 	{{'.', [],
 	  [{'__ENV__', [], 'Elixir.CloudILogger'}, line]},
-	 [], []},
+	 [{no_parens, true}], []},
 	{function, [], 'Elixir.CloudILogger'},
-	{arity, [], 'Elixir.CloudILogger'}, EVformat@1,
-	EVargs@1]}]}.
+	{arity, [], 'Elixir.CloudILogger'}, _Eformat@1,
+	_Eargs@1]}]}.
 
-'MACRO-log_fatal_apply'(_E@CALLER, EVf@1, EVa@1) ->
+'MACRO-log_fatal_apply'(_E@CALLER, _Ef@1, _Ea@1) ->
     {{'.', [],
       [cloudi_core_i_logger_interface, fatal_apply]},
-     [], [EVf@1, EVa@1]}.
+     [], [_Ef@1, _Ea@1]}.
 
-'MACRO-log_fatal_apply'(_E@CALLER, EVm@1, EVf@1,
-			EVa@1) ->
+'MACRO-log_fatal_apply'(_E@CALLER, _Em@1, _Ef@1,
+			_Ea@1) ->
     {{'.', [],
       [cloudi_core_i_logger_interface, fatal_apply]},
-     [], [EVm@1, EVf@1, EVa@1]}.
+     [], [_Em@1, _Ef@1, _Ea@1]}.
 
-'MACRO-log_fatal_sync'(_E@CALLER, EVformat@1,
-		       EVargs@1) ->
+'MACRO-log_fatal_sync'(_E@CALLER, _Eformat@1,
+		       _Eargs@1) ->
     {'__block__', [],
      [{'=', [],
        [{{function, [], 'Elixir.CloudILogger'},
@@ -250,7 +288,7 @@
 	{'case', [],
 	 [{{'.', [],
 	    [{'__ENV__', [], 'Elixir.CloudILogger'}, function]},
-	   [], []},
+	   [{no_parens, true}], []},
 	  [{do,
 	    [{'->', [], [[nil], {undefined, undefined}]},
 	     {'->', [],
@@ -265,12 +303,12 @@
        [{'__MODULE__', [], 'Elixir.CloudILogger'},
 	{{'.', [],
 	  [{'__ENV__', [], 'Elixir.CloudILogger'}, line]},
-	 [], []},
+	 [{no_parens, true}], []},
 	{function, [], 'Elixir.CloudILogger'},
-	{arity, [], 'Elixir.CloudILogger'}, EVformat@1,
-	EVargs@1]}]}.
+	{arity, [], 'Elixir.CloudILogger'}, _Eformat@1,
+	_Eargs@1]}]}.
 
-'MACRO-log_info'(_E@CALLER, EVformat@1, EVargs@1) ->
+'MACRO-log_info'(_E@CALLER, _Eformat@1, _Eargs@1) ->
     {'__block__', [],
      [{'=', [],
        [{{function, [], 'Elixir.CloudILogger'},
@@ -278,7 +316,7 @@
 	{'case', [],
 	 [{{'.', [],
 	    [{'__ENV__', [], 'Elixir.CloudILogger'}, function]},
-	   [], []},
+	   [{no_parens, true}], []},
 	  [{do,
 	    [{'->', [], [[nil], {undefined, undefined}]},
 	     {'->', [],
@@ -291,24 +329,24 @@
        [{'__MODULE__', [], 'Elixir.CloudILogger'},
 	{{'.', [],
 	  [{'__ENV__', [], 'Elixir.CloudILogger'}, line]},
-	 [], []},
+	 [{no_parens, true}], []},
 	{function, [], 'Elixir.CloudILogger'},
-	{arity, [], 'Elixir.CloudILogger'}, EVformat@1,
-	EVargs@1]}]}.
+	{arity, [], 'Elixir.CloudILogger'}, _Eformat@1,
+	_Eargs@1]}]}.
 
-'MACRO-log_info_apply'(_E@CALLER, EVf@1, EVa@1) ->
+'MACRO-log_info_apply'(_E@CALLER, _Ef@1, _Ea@1) ->
     {{'.', [],
       [cloudi_core_i_logger_interface, info_apply]},
-     [], [EVf@1, EVa@1]}.
+     [], [_Ef@1, _Ea@1]}.
 
-'MACRO-log_info_apply'(_E@CALLER, EVm@1, EVf@1,
-		       EVa@1) ->
+'MACRO-log_info_apply'(_E@CALLER, _Em@1, _Ef@1,
+		       _Ea@1) ->
     {{'.', [],
       [cloudi_core_i_logger_interface, info_apply]},
-     [], [EVm@1, EVf@1, EVa@1]}.
+     [], [_Em@1, _Ef@1, _Ea@1]}.
 
-'MACRO-log_info_sync'(_E@CALLER, EVformat@1,
-		      EVargs@1) ->
+'MACRO-log_info_sync'(_E@CALLER, _Eformat@1,
+		      _Eargs@1) ->
     {'__block__', [],
      [{'=', [],
        [{{function, [], 'Elixir.CloudILogger'},
@@ -316,7 +354,7 @@
 	{'case', [],
 	 [{{'.', [],
 	    [{'__ENV__', [], 'Elixir.CloudILogger'}, function]},
-	   [], []},
+	   [{no_parens, true}], []},
 	  [{do,
 	    [{'->', [], [[nil], {undefined, undefined}]},
 	     {'->', [],
@@ -330,20 +368,21 @@
        [{'__MODULE__', [], 'Elixir.CloudILogger'},
 	{{'.', [],
 	  [{'__ENV__', [], 'Elixir.CloudILogger'}, line]},
-	 [], []},
+	 [{no_parens, true}], []},
 	{function, [], 'Elixir.CloudILogger'},
-	{arity, [], 'Elixir.CloudILogger'}, EVformat@1,
-	EVargs@1]}]}.
+	{arity, [], 'Elixir.CloudILogger'}, _Eformat@1,
+	_Eargs@1]}]}.
 
 'MACRO-log_metadata_get'(_E@CALLER) ->
     {{'.', [], [cloudi_core_i_logger, metadata_get]}, [],
      []}.
 
-'MACRO-log_metadata_set'(_E@CALLER, EVl@1) ->
+'MACRO-log_metadata_set'(_E@CALLER, _El@1) ->
     {{'.', [], [cloudi_core_i_logger, metadata_set]}, [],
-     [EVl@1]}.
+     [_El@1]}.
 
-'MACRO-log_trace'(_E@CALLER, EVformat@1, EVargs@1) ->
+'MACRO-log_sync'(_E@CALLER, _Elevel@1, _Eformat@1,
+		 _Eargs@1) ->
     {'__block__', [],
      [{'=', [],
        [{{function, [], 'Elixir.CloudILogger'},
@@ -351,7 +390,34 @@
 	{'case', [],
 	 [{{'.', [],
 	    [{'__ENV__', [], 'Elixir.CloudILogger'}, function]},
-	   [], []},
+	   [{no_parens, true}], []},
+	  [{do,
+	    [{'->', [], [[nil], {undefined, undefined}]},
+	     {'->', [],
+	      [[{'=', [],
+		 [{{'_', [], 'Elixir.CloudILogger'},
+		   {'_', [], 'Elixir.CloudILogger'}},
+		  {function_arity, [], 'Elixir.CloudILogger'}]}],
+	       {function_arity, [], 'Elixir.CloudILogger'}]}]}]]}]},
+      {{'.', [], [cloudi_core_i_logger_interface, log_sync]},
+       [],
+       [{'__MODULE__', [], 'Elixir.CloudILogger'},
+	{{'.', [],
+	  [{'__ENV__', [], 'Elixir.CloudILogger'}, line]},
+	 [{no_parens, true}], []},
+	{function, [], 'Elixir.CloudILogger'},
+	{arity, [], 'Elixir.CloudILogger'}, _Elevel@1,
+	_Eformat@1, _Eargs@1]}]}.
+
+'MACRO-log_trace'(_E@CALLER, _Eformat@1, _Eargs@1) ->
+    {'__block__', [],
+     [{'=', [],
+       [{{function, [], 'Elixir.CloudILogger'},
+	 {arity, [], 'Elixir.CloudILogger'}},
+	{'case', [],
+	 [{{'.', [],
+	    [{'__ENV__', [], 'Elixir.CloudILogger'}, function]},
+	   [{no_parens, true}], []},
 	  [{do,
 	    [{'->', [], [[nil], {undefined, undefined}]},
 	     {'->', [],
@@ -364,24 +430,24 @@
        [{'__MODULE__', [], 'Elixir.CloudILogger'},
 	{{'.', [],
 	  [{'__ENV__', [], 'Elixir.CloudILogger'}, line]},
-	 [], []},
+	 [{no_parens, true}], []},
 	{function, [], 'Elixir.CloudILogger'},
-	{arity, [], 'Elixir.CloudILogger'}, EVformat@1,
-	EVargs@1]}]}.
+	{arity, [], 'Elixir.CloudILogger'}, _Eformat@1,
+	_Eargs@1]}]}.
 
-'MACRO-log_trace_apply'(_E@CALLER, EVf@1, EVa@1) ->
+'MACRO-log_trace_apply'(_E@CALLER, _Ef@1, _Ea@1) ->
     {{'.', [],
       [cloudi_core_i_logger_interface, trace_apply]},
-     [], [EVf@1, EVa@1]}.
+     [], [_Ef@1, _Ea@1]}.
 
-'MACRO-log_trace_apply'(_E@CALLER, EVm@1, EVf@1,
-			EVa@1) ->
+'MACRO-log_trace_apply'(_E@CALLER, _Em@1, _Ef@1,
+			_Ea@1) ->
     {{'.', [],
       [cloudi_core_i_logger_interface, trace_apply]},
-     [], [EVm@1, EVf@1, EVa@1]}.
+     [], [_Em@1, _Ef@1, _Ea@1]}.
 
-'MACRO-log_trace_sync'(_E@CALLER, EVformat@1,
-		       EVargs@1) ->
+'MACRO-log_trace_sync'(_E@CALLER, _Eformat@1,
+		       _Eargs@1) ->
     {'__block__', [],
      [{'=', [],
        [{{function, [], 'Elixir.CloudILogger'},
@@ -389,7 +455,7 @@
 	{'case', [],
 	 [{{'.', [],
 	    [{'__ENV__', [], 'Elixir.CloudILogger'}, function]},
-	   [], []},
+	   [{no_parens, true}], []},
 	  [{do,
 	    [{'->', [], [[nil], {undefined, undefined}]},
 	     {'->', [],
@@ -404,12 +470,12 @@
        [{'__MODULE__', [], 'Elixir.CloudILogger'},
 	{{'.', [],
 	  [{'__ENV__', [], 'Elixir.CloudILogger'}, line]},
-	 [], []},
+	 [{no_parens, true}], []},
 	{function, [], 'Elixir.CloudILogger'},
-	{arity, [], 'Elixir.CloudILogger'}, EVformat@1,
-	EVargs@1]}]}.
+	{arity, [], 'Elixir.CloudILogger'}, _Eformat@1,
+	_Eargs@1]}]}.
 
-'MACRO-log_warn'(_E@CALLER, EVformat@1, EVargs@1) ->
+'MACRO-log_warn'(_E@CALLER, _Eformat@1, _Eargs@1) ->
     {'__block__', [],
      [{'=', [],
        [{{function, [], 'Elixir.CloudILogger'},
@@ -417,7 +483,7 @@
 	{'case', [],
 	 [{{'.', [],
 	    [{'__ENV__', [], 'Elixir.CloudILogger'}, function]},
-	   [], []},
+	   [{no_parens, true}], []},
 	  [{do,
 	    [{'->', [], [[nil], {undefined, undefined}]},
 	     {'->', [],
@@ -430,24 +496,24 @@
        [{'__MODULE__', [], 'Elixir.CloudILogger'},
 	{{'.', [],
 	  [{'__ENV__', [], 'Elixir.CloudILogger'}, line]},
-	 [], []},
+	 [{no_parens, true}], []},
 	{function, [], 'Elixir.CloudILogger'},
-	{arity, [], 'Elixir.CloudILogger'}, EVformat@1,
-	EVargs@1]}]}.
+	{arity, [], 'Elixir.CloudILogger'}, _Eformat@1,
+	_Eargs@1]}]}.
 
-'MACRO-log_warn_apply'(_E@CALLER, EVf@1, EVa@1) ->
+'MACRO-log_warn_apply'(_E@CALLER, _Ef@1, _Ea@1) ->
     {{'.', [],
       [cloudi_core_i_logger_interface, warn_apply]},
-     [], [EVf@1, EVa@1]}.
+     [], [_Ef@1, _Ea@1]}.
 
-'MACRO-log_warn_apply'(_E@CALLER, EVm@1, EVf@1,
-		       EVa@1) ->
+'MACRO-log_warn_apply'(_E@CALLER, _Em@1, _Ef@1,
+		       _Ea@1) ->
     {{'.', [],
       [cloudi_core_i_logger_interface, warn_apply]},
-     [], [EVm@1, EVf@1, EVa@1]}.
+     [], [_Em@1, _Ef@1, _Ea@1]}.
 
-'MACRO-log_warn_sync'(_E@CALLER, EVformat@1,
-		      EVargs@1) ->
+'MACRO-log_warn_sync'(_E@CALLER, _Eformat@1,
+		      _Eargs@1) ->
     {'__block__', [],
      [{'=', [],
        [{{function, [], 'Elixir.CloudILogger'},
@@ -455,7 +521,7 @@
 	{'case', [],
 	 [{{'.', [],
 	    [{'__ENV__', [], 'Elixir.CloudILogger'}, function]},
-	   [], []},
+	   [{no_parens, true}], []},
 	  [{do,
 	    [{'->', [], [[nil], {undefined, undefined}]},
 	     {'->', [],
@@ -469,7 +535,7 @@
        [{'__MODULE__', [], 'Elixir.CloudILogger'},
 	{{'.', [],
 	  [{'__ENV__', [], 'Elixir.CloudILogger'}, line]},
-	 [], []},
+	 [{no_parens, true}], []},
 	{function, [], 'Elixir.CloudILogger'},
-	{arity, [], 'Elixir.CloudILogger'}, EVformat@1,
-	EVargs@1]}]}.
\ No newline at end of file
+	{arity, [], 'Elixir.CloudILogger'}, _Eformat@1,
+	_Eargs@1]}]}.
\ No newline at end of file
