@@ -31,7 +31,7 @@
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
 %%% @copyright 2011-2020 Michael Truog
-%%% @version 1.8.1 {@date} {@time}
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_nodes).
@@ -85,9 +85,6 @@
         log_reconnect :: cloudi_service_api:loglevel()
     }).
 
--define(CATCH_EXIT(F),
-        try F catch exit:{Reason, _} -> {error, Reason} end).
-
 %%%------------------------------------------------------------------------
 %%% External interface functions
 %%%------------------------------------------------------------------------
@@ -582,8 +579,8 @@ nodes_status([Node | NodesSelection], StatusList, TimeNow,
                 NanoSecondsYearDisconnect > 0 ->
                     [{downtime_year_disconnected,
                       cloudi_core_i_status:
-                      nanoseconds_to_string(NanoSecondsYearDisconnect,
-                                            ApproximateYearDisconnect)} |
+                      nanoseconds_to_string_gt(NanoSecondsYearDisconnect,
+                                               ApproximateYearDisconnect)} |
                      Status4];
                 true ->
                     Status4
@@ -594,8 +591,8 @@ nodes_status([Node | NodesSelection], StatusList, TimeNow,
                 NanoSecondsYearDisconnect > 0 ->
                     [{downtime_month_disconnected,
                       cloudi_core_i_status:
-                      nanoseconds_to_string(NanoSecondsMonthDisconnect,
-                                            ApproximateMonthDisconnect)} |
+                      nanoseconds_to_string_gt(NanoSecondsMonthDisconnect,
+                                               ApproximateMonthDisconnect)} |
                      Status5];
                 true ->
                     Status5
@@ -607,8 +604,8 @@ nodes_status([Node | NodesSelection], StatusList, TimeNow,
                 NanoSecondsYearDisconnect > 0 ->
                     [{downtime_week_disconnected,
                       cloudi_core_i_status:
-                      nanoseconds_to_string(NanoSecondsWeekDisconnect,
-                                            ApproximateWeekDisconnect)} |
+                      nanoseconds_to_string_gt(NanoSecondsWeekDisconnect,
+                                               ApproximateWeekDisconnect)} |
                      Status6];
                 true ->
                     Status6
@@ -619,8 +616,8 @@ nodes_status([Node | NodesSelection], StatusList, TimeNow,
                 NanoSecondsMonthDisconnect > 0 ->
                     [{downtime_day_disconnected,
                       cloudi_core_i_status:
-                      nanoseconds_to_string(NanoSecondsDayDisconnect,
-                                            ApproximateDayDisconnect)} |
+                      nanoseconds_to_string_gt(NanoSecondsDayDisconnect,
+                                               ApproximateDayDisconnect)} |
                      Status7];
                 true ->
                     Status7
