@@ -7,49 +7,35 @@
 %%% Routines for live-service testing during development.
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2013-2017, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2013-2017 Michael Truog
-%%% @version 1.5.5 {@date} {@time}
+%%% Copyright (c) 2013-2020 Michael Truog <mjtruog at protonmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
+%%%
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2013-2020 Michael Truog
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_runtime_testing).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 %% external interface
 -export([monkey_latency_format/1,
@@ -65,7 +51,6 @@
 -include("cloudi_logger.hrl").
 -include("cloudi_core_i_constants.hrl").
 
--define(DAY_MILLISECONDS, (24 * 60 * 60 * 1000)).
 -define(MONKEY_LATENCY_METHOD_DEFAULT, time_absolute).
 -define(MONKEY_LATENCY_MEAN_DEFAULT, 5000). % milliseconds
 -define(MONKEY_LATENCY_LOG, 5000). % milliseconds
@@ -93,7 +78,8 @@
 %%%------------------------------------------------------------------------
 
 -spec monkey_latency_format(#monkey_latency{} | system | false) ->
-    list({atom(), any()}) | system | false.
+    cloudi_service_api:service_options_monkey_latency_options() |
+    system | false.
 
 monkey_latency_format(false) ->
     false;
@@ -142,24 +128,16 @@ monkey_latency_init(#monkey_latency{} = MonkeyLatency) ->
 monkey_latency_check(#monkey_latency{method = time_uniform,
                                      value1 = Min,
                                      value2 = Max} = MonkeyLatency) ->
-    Result = crypto:rand_uniform(Min, Max + 1),
+    Result = random_range(Min, Max),
     sleep(Result),
     MonkeyLatency;
 monkey_latency_check(#monkey_latency{method = time_gaussian,
                                      value1 = Mean,
                                      value2 = StdDev,
-                                     result1 = undefined,
-                                     pi2 = PI2} = MonkeyLatency) ->
-    % use Box-Muller transformation to generate Gaussian noise
-    % (G. E. P. Box and Mervin E. Muller,
-    %  A Note on the Generation of Random Normal Deviates,
-    %  The Annals of Mathematical Statistics (1958),
-    %  Vol. 29, No. 2 pp. 610–611)
-    X1 = random(),
-    X2 = PI2 * random(),
-    K = StdDev * math:sqrt(-2.0 * math:log(X1)),
-    Result1 = erlang:max(erlang:round(Mean + K * math:cos(X2)), 1),
-    Result2 = erlang:max(erlang:round(Mean + K * math:sin(X2)), 1),
+                                     result1 = undefined} = MonkeyLatency) ->
+    {Latency1, Latency2} = cloudi_x_quickrand_normal:box_muller(Mean, StdDev),
+    Result1 = erlang:max(erlang:round(Latency1), 1),
+    Result2 = erlang:max(erlang:round(Latency2), 1),
     sleep(Result2),
     MonkeyLatency#monkey_latency{result1 = Result1};
 monkey_latency_check(#monkey_latency{method = time_gaussian,
@@ -173,7 +151,8 @@ monkey_latency_check(#monkey_latency{method = time_absolute,
     MonkeyLatency.
 
 -spec monkey_chaos_format(#monkey_chaos{} | system | false) ->
-    list({atom(), any()}) | system | false.
+    cloudi_service_api:service_options_monkey_chaos_options() |
+    system | false.
 
 monkey_chaos_format(false) ->
     false;
@@ -227,7 +206,7 @@ monkey_chaos_check(#monkey_chaos{method = probability_day,
                                  value1 = Percent,
                                  pid = undefined} = MonkeyChaos) ->
     Pid = erlang:spawn_link(fun() ->
-        erlang:process_flag(trap_exit, true),
+        false = erlang:process_flag(trap_exit, true),
         monkey_chaos_pid_day(Percent)
     end),
     MonkeyChaos#monkey_chaos{pid = Pid};
@@ -427,9 +406,9 @@ monkey_chaos_pid_day(Percent)
     DieToday = random() =< Percent,
     Delay = if
         DieToday =:= true ->
-            erlang:round(?DAY_MILLISECONDS * random());
+            erlang:round(?MILLISECONDS_IN_DAY * random());
         DieToday =:= false ->
-            ?DAY_MILLISECONDS
+            ?MILLISECONDS_IN_DAY
     end,
     receive
         'monkey_chaos_destroy' ->
@@ -446,6 +425,9 @@ monkey_chaos_pid_day(Percent)
             end
     end.
 
+random_range(Min, Max) ->
+    cloudi_x_quickrand:strong_uniform_range(Min, Max).
+
 % return a floating point value between 0.0 and 1.0, inclusive
 random() ->
     cloudi_x_quickrand:strong_float().
