@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2009-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2009-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,13 +28,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2009-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2009-2020 Michael Truog
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_sup).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(supervisor).
 
@@ -91,14 +91,17 @@ start_link(Config) when is_record(Config, config) ->
 -define(CHECK,
         (is_tuple(child_specification(cloudi_core_i_services_external_sup))
          andalso
-         is_tuple(child_specification(cloudi_core_i_os_spawn_pool)))).
+         is_tuple(child_specification(?OS_SPAWN_POOL))
+         andalso
+         is_tuple(child_specification(?OS_COMMAND_POOL)))).
 -else.
 -define(CHILDSPECS,
         [child_specification(cloudi_core_i_logger_sup, Config),
          child_specification(cloudi_core_i_logger, Config),
          child_specification(cloudi_core_i_nodes, Config),
          child_specification(cloudi_core_i_services_monitor),
-         child_specification(cloudi_core_i_os_spawn_pool),
+         child_specification(?OS_SPAWN_POOL),
+         child_specification(?OS_COMMAND_POOL),
          child_specification(cloudi_core_i_services_external_sup),
          child_specification(cloudi_core_i_services_internal_sup),
          child_specification(cloudi_core_i_configurator, Config),
@@ -137,7 +140,7 @@ child_specification(cloudi_core_i_nodes, Config) ->
 child_specification(cloudi_core_i_configurator, Config) ->
     Shutdown = 2000, % milliseconds
     {cloudi_core_i_configurator,
-     {cloudi_core_i_configurator, start_link, [Config]},
+     {cloudi_core_i_configurator, start_link, [Shutdown, Config]},
      permanent, Shutdown, worker, [cloudi_core_i_configurator]}.
 
 child_specification(cloudi_core_i_services_monitor) ->
@@ -157,26 +160,29 @@ child_specification(cloudi_core_i_services_internal_sup) ->
      {cloudi_core_i_services_internal_sup, start_link, []},
      permanent, infinity, supervisor, [cloudi_core_i_services_internal_sup]};
 
-child_specification(cloudi_core_i_os_spawn_pool) ->
+child_specification(?OS_SPAWN_POOL = OSSpawnPool) ->
     Shutdown = 2000, % milliseconds
-    {cloudi_core_i_os_spawn_pool,
+    {OSSpawnPool,
      {cloudi_x_supool_sup, start_link,
-      [cloudi_core_i_os_spawn, os_process_count(),
+      [OSSpawnPool, os_process_count(),
        {undefined, {cloudi_core_i_os_spawn, start_link, []},
         permanent, Shutdown, worker, [cloudi_core_i_os_spawn]}, []]},
      permanent, infinity, supervisor, [cloudi_x_supool_sup]};
 
+child_specification(?OS_COMMAND_POOL = OSCommandPool) ->
+    Shutdown = 2000, % milliseconds
+    {OSCommandPool,
+     {cloudi_x_supool_sup, start_link,
+      [OSCommandPool, os_process_count(),
+       {undefined, {cloudi_core_i_os_command, start_link, []},
+        permanent, Shutdown, worker, [cloudi_core_i_os_command]}, []]},
+     permanent, infinity, supervisor, [cloudi_x_supool_sup]};
+
 child_specification(cloudi_core_i_services_external_sup) ->
     {cloudi_core_i_services_external_sup,
      {cloudi_core_i_services_external_sup, start_link, []},
      permanent, infinity, supervisor, [cloudi_core_i_services_external_sup]}.
 
-% determine the number of cloudi_core_i_os_spawn child processes within the
-% cloudi_x_supool_sup, referenced by the locally registered name (of the pool):
-% 'cloudi_core_i_os_spawn'
-
-%os_process_count() ->
-%    1.
 os_process_count() ->
     erlang:system_info(schedulers).
 
