@@ -6,49 +6,35 @@
 %%% ==CloudI PostgreSQL Data Module==
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2009-2015, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2009-2015 Michael Truog
-%%% @version 1.5.1 {@date} {@time}
+%%% Copyright (c) 2009-2020 Michael Truog <mjtruog at protonmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
+%%%
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2009-2020 Michael Truog
+%%% @version 1.8.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_db_pgsql).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(cloudi_service).
 
@@ -66,10 +52,9 @@
 
 -include_lib("cloudi_core/include/cloudi_logger.hrl").
 -include_lib("cloudi_x_epgsql/include/cloudi_x_epgsql.hrl").
--include_lib("cloudi_x_epgsql_wg/include/cloudi_x_epgsql_wg.hrl").
 
 -define(DEFAULT_DATABASE,               undefined). % required argument, string
--define(DEFAULT_DRIVER,                        wg).
+-define(DEFAULT_DRIVER,                 semiocast).
 -define(DEFAULT_HOST_NAME,            "127.0.0.1").
 -define(DEFAULT_USER_NAME,               "cloudi").
 -define(DEFAULT_PASSWORD,                      "").
@@ -85,14 +70,29 @@
 -define(DEFAULT_DEBUG,                      false). % log output for debugging
 -define(DEFAULT_DEBUG_LEVEL,                trace).
 
+
 % supported drivers
 -define(MODULE_EPGSQL, cloudi_x_epgsql).
--define(MODULE_WG, cloudi_x_epgsql_wg). % default
 -define(MODULE_SEMIOCAST, cloudi_x_pgsql_connection).
 
+-define(DRIVER_DEBUG(Level, Query, Parameters, Result),
+    ?LOG(Level,
+         "SQL(equery):~n"
+         " ~p~n"
+         " ~p~n"
+         " = ~p",
+         [Query, Parameters, Result])).
+
+-define(DRIVER_DEBUG(Level, Query, Result),
+    ?LOG(Level,
+         "SQL(squery):~n"
+         " ~p~n"
+         " = ~p",
+         [Query, Result])).
+
 -record(state,
     {
-        module :: ?MODULE_WG | ?MODULE_SEMIOCAST | ?MODULE_EPGSQL,
+        module :: ?MODULE_EPGSQL | ?MODULE_SEMIOCAST,
         connection :: any(),
         listen :: cloudi_service:service_name() | undefined,
         output_type :: both | internal | external,
@@ -119,7 +119,7 @@
 -type timeout_milliseconds() :: cloudi:timeout_milliseconds().
 -type module_response(Result) ::
     {{ok, Result}, NewAgent :: agent()} |
-    {{error, cloudi:error_reason_sync()}, NewAgent :: agent()}.
+    {{error, cloudi:error_reason()}, NewAgent :: agent()}.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -265,9 +265,6 @@ cloudi_service_init(Args, _Prefix, Timeout, Dispatcher) ->
     Module = if
         Driver =:= epgsql ->
             ?MODULE_EPGSQL;
-        Driver =:= wg; Driver =:= epgsql_wg ->
-            % NewArgs is passed to ssl application
-            ?MODULE_WG;
         Driver =:= semiocast ->
             [] = NewArgs,
             ?MODULE_SEMIOCAST
@@ -329,7 +326,8 @@ cloudi_service_init(Args, _Prefix, Timeout, Dispatcher) ->
             {stop, Reason, undefined}
     end.
 
-cloudi_service_handle_request(_Type, _Name, _Pattern, _RequestInfo, Request,
+cloudi_service_handle_request(_RequestType, _Name, _Pattern,
+                              _RequestInfo, Request,
                               Timeout, _Priority, _TransId, _Pid,
                               #state{output_type = OutputType,
                                      external_format = ExternalFormat} = State,
@@ -380,11 +378,9 @@ cloudi_service_handle_info({AsyncTag, Connection, Async},
     {noreply, State};
 
 cloudi_service_handle_info(Request, State, _Dispatcher) ->
-    ?LOG_WARN("Unknown info \"~p\"", [Request]),
-    {noreply, State}.
+    {stop, cloudi_string:format("Unknown info \"~w\"", [Request]), State}.
 
-cloudi_service_terminate(_Reason, _Timeout,
-                         undefined) ->
+cloudi_service_terminate(_Reason, _Timeout, undefined) ->
     ok;
 cloudi_service_terminate(_Reason, _Timeout,
                          #state{module = Module,
@@ -573,8 +569,6 @@ mysql_query_transform_get(NewString, Index, Remaining) ->
 
 driver_open(?MODULE_EPGSQL, HostName, UserName, Password, Args) ->
     ?MODULE_EPGSQL:connect(HostName, UserName, Password, Args);
-driver_open(?MODULE_WG, HostName, UserName, Password, Args) ->
-    ?MODULE_WG:connect(HostName, UserName, Password, Args);
 driver_open(?MODULE_SEMIOCAST, HostName, UserName, Password, Args0) ->
     {value, {_, Database}, Args1} = lists:keytake(database, 1, Args0),
     % timeout is per function call (not open though)
@@ -590,8 +584,6 @@ driver_open(?MODULE_SEMIOCAST, HostName, UserName, Password, Args0) ->
 
 driver_close(?MODULE_EPGSQL, Connection) ->
     ?MODULE_EPGSQL:close(Connection);
-driver_close(?MODULE_WG, Connection) ->
-    ?MODULE_WG:close(Connection);
 driver_close(?MODULE_SEMIOCAST, Connection) ->
     ?MODULE_SEMIOCAST:close(Connection).
 
@@ -626,12 +618,7 @@ driver_equery(Query, Parameters, _Timeout, ResponseOutputType,
                      interface = Interface,
                      debug_level = DebugLevel}) ->
     Native = ?MODULE_EPGSQL:equery(Connection, Query, Parameters),
-    if
-        DebugLevel =:= off ->
-            ok;
-        true ->
-            driver_debug(DebugLevel, Query, Parameters, Native)
-    end,
+    ?DRIVER_DEBUG(DebugLevel, Query, Parameters, Native),
     if
         Native == {error, closed} ->
             erlang:exit(closed);
@@ -650,37 +637,6 @@ driver_equery(Query, Parameters, _Timeout, ResponseOutputType,
         ResponseOutputType =:= external ->
             response_external(Response, ExternalFormat)
     end;
-driver_equery(Query, Parameters, _Timeout, ResponseOutputType,
-              #state{module = ?MODULE_WG,
-                     connection = Connection,
-                     external_format = ExternalFormat,
-                     interface = Interface,
-                     debug_level = DebugLevel}) ->
-    Native = ?MODULE_WG:equery(Connection, Query, Parameters),
-    if
-        DebugLevel =:= off ->
-            ok;
-        true ->
-            driver_debug(DebugLevel, Query, Parameters, Native)
-    end,
-    if
-        Native == {error, closed} ->
-            erlang:exit(closed);
-        true ->
-            ok
-    end,
-    Response = if
-        Interface =:= common; ResponseOutputType =:= external ->
-            wg_to_common(Native);
-        Interface =:= native ->
-            Native
-    end,
-    if
-        ResponseOutputType =:= internal ->
-            Response;
-        ResponseOutputType =:= external ->
-            response_external(Response, ExternalFormat)
-    end;
 driver_equery(Query, Parameters, Timeout, ResponseOutputType,
               #state{module = ?MODULE_SEMIOCAST,
                      connection = Connection,
@@ -689,12 +645,7 @@ driver_equery(Query, Parameters, Timeout, ResponseOutputType,
                      debug_level = DebugLevel}) ->
     Native = ?MODULE_SEMIOCAST:extended_query(Query, Parameters, [],
                                               Timeout, Connection),
-    if
-        DebugLevel =:= off ->
-            ok;
-        true ->
-            driver_debug(DebugLevel, Query, Parameters, Native)
-    end,
+    ?DRIVER_DEBUG(DebugLevel, Query, Parameters, Native),
     if
         Native == {error, closed} ->
             erlang:exit(closed);
@@ -721,12 +672,7 @@ driver_squery(Query, _Timeout, ResponseOutputType,
                      interface = Interface,
                      debug_level = DebugLevel}) ->
     Native = ?MODULE_EPGSQL:squery(Connection, Query),
-    if
-        DebugLevel =:= off ->
-            ok;
-        true ->
-            driver_debug(DebugLevel, Query, Native)
-    end,
+    ?DRIVER_DEBUG(DebugLevel, Query, Native),
     if
         Native == {error, closed} ->
             erlang:exit(closed);
@@ -745,37 +691,6 @@ driver_squery(Query, _Timeout, ResponseOutputType,
         ResponseOutputType =:= external ->
             response_external(Response, ExternalFormat)
     end;
-driver_squery(Query, _Timeout, ResponseOutputType,
-              #state{module = ?MODULE_WG,
-                     connection = Connection,
-                     external_format = ExternalFormat,
-                     interface = Interface,
-                     debug_level = DebugLevel}) ->
-    Native = ?MODULE_WG:squery(Connection, Query),
-    if
-        DebugLevel =:= off ->
-            ok;
-        true ->
-            driver_debug(DebugLevel, Query, Native)
-    end,
-    if
-        Native == {error, closed} ->
-            erlang:exit(closed);
-        true ->
-            ok
-    end,
-    Response = if
-        Interface =:= common; ResponseOutputType =:= external ->
-            wg_to_common(Native);
-        Interface =:= native ->
-            Native
-    end,
-    if
-        ResponseOutputType =:= internal ->
-            Response;
-        ResponseOutputType =:= external ->
-            response_external(Response, ExternalFormat)
-    end;
 driver_squery(Query, Timeout, ResponseOutputType,
               #state{module = ?MODULE_SEMIOCAST,
                      connection = Connection,
@@ -783,12 +698,7 @@ driver_squery(Query, Timeout, ResponseOutputType,
                      interface = Interface,
                      debug_level = DebugLevel}) ->
     Native = ?MODULE_SEMIOCAST:simple_query(Query, [], Timeout, Connection),
-    if
-        DebugLevel =:= off ->
-            ok;
-        true ->
-            driver_debug(DebugLevel, Query, Native)
-    end,
+    ?DRIVER_DEBUG(DebugLevel, Query, Native),
     if
         Native == {error, closed} ->
             erlang:exit(closed);
@@ -808,34 +718,6 @@ driver_squery(Query, Timeout, ResponseOutputType,
             response_external(Response, ExternalFormat)
     end.
 
-driver_debug_log(trace, Message, Args) ->
-    ?LOG_TRACE(Message, Args);
-driver_debug_log(debug, Message, Args) ->
-    ?LOG_DEBUG(Message, Args);
-driver_debug_log(info, Message, Args) ->
-    ?LOG_INFO(Message, Args);
-driver_debug_log(warn, Message, Args) ->
-    ?LOG_WARN(Message, Args);
-driver_debug_log(error, Message, Args) ->
-    ?LOG_ERROR(Message, Args);
-driver_debug_log(fatal, Message, Args) ->
-    ?LOG_FATAL(Message, Args).
-
-driver_debug(Level, Query, Parameters, Result) ->
-    driver_debug_log(Level,
-                     "SQL(equery):~n"
-                     " ~p~n"
-                     " ~p~n"
-                     " = ~p",
-                     [Query, Parameters, Result]).
-
-driver_debug(Level, Query, Result) ->
-    driver_debug_log(Level,
-                     "SQL(squery):~n"
-                     " ~p~n"
-                     " = ~p",
-                     [Query, Result]).
-
 epgsql_to_common({ok, I}) ->
     {updated, I};
 epgsql_to_common({ok, [], []}) ->
@@ -849,20 +731,6 @@ epgsql_to_common({error, #error{message = Message}}) ->
 epgsql_to_common([_ | _] = L) ->
     check_list(L, fun epgsql_to_common/1).
 
-% Rows in the wg format only use binary strings for data
-wg_to_common({ok, I}) ->
-    {updated, I};
-wg_to_common({ok, [], []}) ->
-    {updated, 0};
-wg_to_common({ok, _Columns, Rows}) ->
-    {selected, Rows};
-wg_to_common({ok, I, _Columns, Rows}) ->
-    {updated, I, Rows};
-wg_to_common({error, #epgsql_wg_error{message = Message}}) ->
-    {error, Message};
-wg_to_common([_ | _] = L) ->
-    check_list(L, fun wg_to_common/1).
-
 semiocast_to_common_rows([] = Rows) ->
     Rows;
 semiocast_to_common_rows([Row | _] = Rows) ->
@@ -946,12 +814,6 @@ driver_async_to_common(?MODULE_EPGSQL,
 driver_async_to_common(?MODULE_EPGSQL,
                        {notice, #error{message = Message}}) ->
     {notify_error, Message};
-driver_async_to_common(?MODULE_WG,
-                       {notification, Channel, Pid, Payload}) ->
-    {notify_ok, Pid, Channel, Payload};
-driver_async_to_common(?MODULE_WG,
-                       {notice, #epgsql_wg_error{message = Message}}) ->
-    {notify_error, Message};
 driver_async_to_common(?MODULE_SEMIOCAST,
                        {notification, Pid, Channel, Payload}) ->
     {notify_ok, Pid, Channel, Payload};
