@@ -24,7 +24,7 @@
 %%%
 %%% @author Loïc Hoguin <essen@ninenines.eu>
 %%% @copyright 2011-2014 Loïc Hoguin
-%%% @version 1.7.1 {@date} {@time}
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_filesystem_parse).
@@ -52,9 +52,45 @@ range(Binary) ->
     token_ci(Binary, fun range/2).
 
 %%%------------------------------------------------------------------------
-%%% From cowboy, in cowboy_http.erl
+%%% From cowboy1, in cowboy1_bstr.erl
 %%%------------------------------------------------------------------------
 
+-spec char_to_lower(byte()) -> byte().
+char_to_lower($A) -> $a;
+char_to_lower($B) -> $b;
+char_to_lower($C) -> $c;
+char_to_lower($D) -> $d;
+char_to_lower($E) -> $e;
+char_to_lower($F) -> $f;
+char_to_lower($G) -> $g;
+char_to_lower($H) -> $h;
+char_to_lower($I) -> $i;
+char_to_lower($J) -> $j;
+char_to_lower($K) -> $k;
+char_to_lower($L) -> $l;
+char_to_lower($M) -> $m;
+char_to_lower($N) -> $n;
+char_to_lower($O) -> $o;
+char_to_lower($P) -> $p;
+char_to_lower($Q) -> $q;
+char_to_lower($R) -> $r;
+char_to_lower($S) -> $s;
+char_to_lower($T) -> $t;
+char_to_lower($U) -> $u;
+char_to_lower($V) -> $v;
+char_to_lower($W) -> $w;
+char_to_lower($X) -> $x;
+char_to_lower($Y) -> $y;
+char_to_lower($Z) -> $z;
+char_to_lower(Ch) -> Ch.
+
+%%%------------------------------------------------------------------------
+%%% From cowboy1, in cowboy1_http.erl
+%%%------------------------------------------------------------------------
+
+-type datetime_fun() ::
+    fun((_, _) -> calendar:datetime() | {error, badarg}).
+
 %% @doc Parse an HTTP date (RFC1123, RFC850 or asctime date).
 %% @end
 %%
@@ -181,7 +217,7 @@ wkday(_Any, _Fun) ->
     {error, badarg}.
 
 %% We never use it, pretty much just checks the weekday is right.
--spec weekday(binary(), fun()) -> any().
+-spec weekday(binary(), datetime_fun()) -> any().
 weekday(<< "Monday", Rest/binary >>, Fun) ->
     Fun(Rest, <<"Monday">>);
 weekday(<< "Tuesday", Rest/binary >>, Fun) ->
@@ -199,7 +235,7 @@ weekday(<< "Sunday", Rest/binary >>, Fun) ->
 weekday(_Any, _Fun) ->
     {error, badarg}.
 
--spec date1(binary(), fun()) -> any().
+-spec date1(binary(), datetime_fun()) -> calendar:datetime() | {error, badarg}.
 date1(<< D1, D2, " ", M:3/binary, " ", Y1, Y2, Y3, Y4, Rest/binary >>, Fun)
         when D1 >= $0, D1 =< $9, D2 >= $0, D2 =< $9,
              Y1 >= $0, Y1 =< $9, Y2 >= $0, Y2 =< $9,
@@ -217,7 +253,7 @@ date1(<< D1, D2, " ", M:3/binary, " ", Y1, Y2, Y3, Y4, Rest/binary >>, Fun)
 date1(_Data, _Fun) ->
     {error, badarg}.
 
--spec date2(binary(), fun()) -> any().
+-spec date2(binary(), datetime_fun()) -> calendar:datetime() | {error, badarg}.
 date2(<< D1, D2, "-", M:3/binary, "-", Y1, Y2, Rest/binary >>, Fun)
         when D1 >= $0, D1 =< $9, D2 >= $0, D2 =< $9,
              Y1 >= $0, Y1 =< $9, Y2 >= $0, Y2 =< $9 ->
@@ -239,7 +275,7 @@ date2(<< D1, D2, "-", M:3/binary, "-", Y1, Y2, Rest/binary >>, Fun)
 date2(_Data, _Fun) ->
     {error, badarg}.
 
--spec date3(binary(), fun()) -> any().
+-spec date3(binary(), datetime_fun()) -> any().
 date3(<< M:3/binary, " ", D1, D2, Rest/binary >>, Fun)
         when (D1 >= $0 andalso D1 =< $3) orelse D1 =:= $\s,
              D2 >= $0, D2 =< $9 ->
@@ -271,7 +307,7 @@ month(<<"Nov">>) -> 11;
 month(<<"Dec">>) -> 12;
 month(_Any) -> {error, badarg}.
 
--spec time(binary(), fun()) -> any().
+-spec time(binary(), datetime_fun()) -> calendar:datetime() | {error, badarg}.
 time(<< H1, H2, ":", M1, M2, ":", S1, S2, Rest/binary >>, Fun)
         when H1 >= $0, H1 =< $2, H2 >= $0, H2 =< $9,
              M1 >= $0, M1 =< $5, M2 >= $0, M2 =< $9,
@@ -327,14 +363,14 @@ range_ending(Data, Fun, RangeBeginning) ->
             {error, badarg}
         end).
 
--spec range_digits(binary(), fun()) -> any().
+-spec range_digits(binary(), fun((_, _) -> any())) -> any().
 range_digits(Data, Fun) ->
     whitespace(Data,
         fun(D) ->
             digits(D, Fun)
         end).
 
--spec range_digits(binary(), any(), fun()) -> any().
+-spec range_digits(binary(), infinity | suffix, fun((_, _) -> any())) -> any().
 range_digits(Data, Default, Fun) ->
     whitespace(Data,
         fun(<< C, Rest/binary >>) when C >= $0, C =< $9 ->
@@ -343,7 +379,7 @@ range_digits(Data, Default, Fun) ->
             Fun(Data, Default)
         end).
 
--spec list(binary(), fun()) -> list() | {error, badarg}.
+-spec list(binary(), fun((_, _) -> any())) -> list() | {error, badarg}.
 list(Data, Fun) ->
     case list(Data, Fun, []) of
         {error, badarg} -> {error, badarg};
@@ -385,7 +421,7 @@ digits(Data, Fun, Acc) ->
     Fun(Data, Acc).
 
 %% Changes all characters to lowercase.
--spec token_ci(binary(), fun()) -> any().
+-spec token_ci(binary(), fun((_, _) -> any())) -> any().
 token_ci(Data, Fun) ->
     token(Data, Fun, ci, <<>>).
 
@@ -422,37 +458,48 @@ whitespace(Data, Fun) ->
 -ifdef(TEST).
 -include_lib("eunit/include/eunit.hrl").
 
-http_date_test_() ->
+-include("cloudi_service_filesystem_test.hrl").
+
+module_test_() ->
+    {timeout, ?TEST_TIMEOUT, [
+        {"http date tests", t_http_date()},
+        {"rfc1123 date tests", t_rfc1123_date()},
+        {"rfc850 date tests", t_rfc850_date()},
+        {"asctime date tests", t_asctime_date()},
+        {"http range tests", t_http_range()}
+    ]}.
+
+t_http_date() ->
     %% {Tokens, Result}
     Tests = [
         {<<"Sun, 06 Nov 1994 08:49:37 GMT">>, {{1994, 11, 6}, {8, 49, 37}}},
         {<<"Sunday, 06-Nov-94 08:49:37 GMT">>, {{1994, 11, 6}, {8, 49, 37}}},
         {<<"Sun Nov  6 08:49:37 1994">>, {{1994, 11, 6}, {8, 49, 37}}}
     ],
-    [{V, fun() -> R = http_date(V) end} || {V, R} <- Tests].
+    [{V, ?_assertEqual(R, http_date(V))} || {V, R} <- Tests].
 
-rfc1123_date_test_() ->
+t_rfc1123_date() ->
     %% {Tokens, Result}
     Tests = [
         {<<"Sun, 06 Nov 1994 08:49:37 GMT">>, {{1994, 11, 6}, {8, 49, 37}}}
     ],
-    [{V, fun() -> R = rfc1123_date(V) end} || {V, R} <- Tests].
+    [{V, ?_assertEqual(R, rfc1123_date(V))} || {V, R} <- Tests].
 
-rfc850_date_test_() ->
+t_rfc850_date() ->
     %% {Tokens, Result}
     Tests = [
         {<<"Sunday, 06-Nov-94 08:49:37 GMT">>, {{1994, 11, 6}, {8, 49, 37}}}
     ],
-    [{V, fun() -> R = rfc850_date(V) end} || {V, R} <- Tests].
+    [{V, ?_assertEqual(R, rfc850_date(V))} || {V, R} <- Tests].
 
-asctime_date_test_() ->
+t_asctime_date() ->
     %% {Tokens, Result}
     Tests = [
         {<<"Sun Nov  6 08:49:37 1994">>, {{1994, 11, 6}, {8, 49, 37}}}
     ],
-    [{V, fun() -> R = asctime_date(V) end} || {V, R} <- Tests].
+    [{V, ?_assertEqual(R, asctime_date(V))} || {V, R} <- Tests].
 
-http_range_test_() ->
+t_http_range() ->
     Tests = [
         {<<"bytes=1-20">>,
             {<<"bytes">>, [{1, 20}]}},
@@ -473,38 +520,7 @@ http_range_test_() ->
         {<<"bytes=-30,-">>,
             {error, badarg}}
     ],
-    [fun() -> R = range(V) end ||{V, R} <- Tests].
+    [?_assertEqual(R, range(V)) || {V, R} <- Tests].
+
 -endif.
 
-%%%------------------------------------------------------------------------
-%%% From cowboy, in cowboy_bstr.erl
-%%%------------------------------------------------------------------------
-
--spec char_to_lower(char()) -> char().
-char_to_lower($A) -> $a;
-char_to_lower($B) -> $b;
-char_to_lower($C) -> $c;
-char_to_lower($D) -> $d;
-char_to_lower($E) -> $e;
-char_to_lower($F) -> $f;
-char_to_lower($G) -> $g;
-char_to_lower($H) -> $h;
-char_to_lower($I) -> $i;
-char_to_lower($J) -> $j;
-char_to_lower($K) -> $k;
-char_to_lower($L) -> $l;
-char_to_lower($M) -> $m;
-char_to_lower($N) -> $n;
-char_to_lower($O) -> $o;
-char_to_lower($P) -> $p;
-char_to_lower($Q) -> $q;
-char_to_lower($R) -> $r;
-char_to_lower($S) -> $s;
-char_to_lower($T) -> $t;
-char_to_lower($U) -> $u;
-char_to_lower($V) -> $v;
-char_to_lower($W) -> $w;
-char_to_lower($X) -> $x;
-char_to_lower($Y) -> $y;
-char_to_lower($Z) -> $z;
-char_to_lower(Ch) -> Ch.
