@@ -10,7 +10,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2012-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2012-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -31,8 +31,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2012-2018 Michael Truog
-%%% @version 1.7.4 {@date} {@time}
+%%% @copyright 2012-2020 Michael Truog
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_map_reduce).
@@ -40,6 +40,12 @@
 
 -behaviour(cloudi_service).
 
+%% external interface
+-export([resume/2,
+         resume/3,
+         suspend/2,
+         suspend/3]).
+
 %% cloudi_service callbacks
 -export([cloudi_service_init/4,
          cloudi_service_handle_request/11,
@@ -51,20 +57,24 @@
 
 -define(DEFAULT_MAP_REDUCE_MODULE,     undefined).
 -define(DEFAULT_MAP_REDUCE_ARGUMENTS,         []).
+-define(DEFAULT_NAME,               "controller").
 -define(DEFAULT_CONCURRENCY,                 1.0). % schedulers multiplier
 -define(DEFAULT_LOG_EXECUTION_TIME,         true).
 
--type map_send_args() :: list().
+-type map_send_args() :: nonempty_list().
 -export_type([map_send_args/0]).
 
 -record(state,
     {
-        time_start :: cloudi_timestamp:seconds_monotonic(),
-        log_execution_time :: boolean(),
         map_reduce_module :: module(),
         map_reduce_state :: any(),
+        map_reduce_name :: string(),
         map_count :: pos_integer(),
-        map_requests :: #{cloudi_service:trans_id() := map_send_args()}
+        log_execution_time :: boolean(),
+        map_requests :: #{cloudi_service:trans_id() := map_send_args()},
+        time_running :: cloudi_timestamp:seconds_monotonic(),
+        suspended = false :: boolean(),
+        elapsed_seconds = 0 :: non_neg_integer()
     }).
 
 -record(init_begin,
@@ -72,11 +82,12 @@
         service :: pid(),
         prefix :: string(),
         timeout :: cloudi_service_api:timeout_initialize_value_milliseconds(),
-        time_start :: cloudi_timestamp:seconds_monotonic(),
-        log_execution_time :: boolean(),
         map_reduce_module :: module(),
         map_reduce_args :: list(),
-        concurrency :: number()
+        map_reduce_name :: string(),
+        concurrency :: number(),
+        log_execution_time :: boolean(),
+        time_start :: cloudi_timestamp:seconds_monotonic()
     }).
 
 -record(init_end,
@@ -87,10 +98,6 @@
 
 -define(INFO_RETRY_INTERVAL, 500). % milliseconds
 
-%%%------------------------------------------------------------------------
-%%% External interface functions
-%%%------------------------------------------------------------------------
-
 %%%------------------------------------------------------------------------
 %%% Callback functions from behavior
 %%%------------------------------------------------------------------------
@@ -107,13 +114,13 @@
 
 -callback cloudi_service_map_reduce_send(ModuleReduceState :: any(),
                                          Dispatcher :: pid()) ->
-    {'ok', SendArgs :: map_send_args(), NewModuleReduceState :: any()} |
-    {'done', NewModuleReduceState :: any()} |
+    {'ok', SendArgs :: map_send_args(), ModuleReduceStateNew :: any()} |
+    {'done', ModuleReduceStateNew :: any()} |
     {'error', Reason :: any()}.
 
 -callback cloudi_service_map_reduce_resend(SendArgs :: map_send_args(),
                                            ModuleReduceState :: any()) ->
-    {'ok', NewSendArgs :: map_send_args(), NewModuleReduceState :: any()} |
+    {'ok', SendArgsNew :: map_send_args(), ModuleReduceStateNew :: any()} |
     {'error', Reason :: any()}.
 
 -callback cloudi_service_map_reduce_recv(SendArgs :: map_send_args(),
@@ -123,28 +130,70 @@
                                          TransId :: binary(),
                                          ModuleReduceState :: any(),
                                          Dispatcher :: pid()) ->
-    {'ok', NewModuleReduceState :: any()} |
-    {'done', NewModuleReduceState :: any()} |
+    {'ok', ModuleReduceStateNew :: any()} |
+    {'done', ModuleReduceStateNew :: any()} |
     {'error', Reason :: any()}.
 
 -callback cloudi_service_map_reduce_info(Request :: any(),
                                          ModuleReduceState :: any(),
                                          Dispatcher :: pid()) ->
-    {'ok', NewModuleReduceState :: any()} |
-    {'done', NewModuleReduceState :: any()} |
+    {'ok', ModuleReduceStateNew :: any()} |
+    {'done', ModuleReduceStateNew :: any()} |
     {'error', Reason :: any()}.
 
+%%%------------------------------------------------------------------------
+%%% External interface functions
+%%%------------------------------------------------------------------------
+
+-type agent() :: cloudi:agent().
+-type service_name() :: cloudi:service_name().
+-type timeout_milliseconds() :: cloudi:timeout_milliseconds().
+-type module_response(Result) ::
+    {{ok, Result}, AgentNew :: agent()} |
+    {{error, cloudi:error_reason()}, AgentNew :: agent()}.
+
+-spec resume(Agent :: agent(),
+             Prefix :: service_name()) ->
+    module_response(ok | {error, any()}).
+
+resume(Agent, Prefix) ->
+    cloudi:send_sync(Agent, Prefix, resume).
+
+-spec resume(Agent :: agent(),
+             Prefix :: service_name(),
+             Timeout :: timeout_milliseconds()) ->
+    module_response(ok | {error, any()}).
+
+resume(Agent, Prefix, Timeout) ->
+    cloudi:send_sync(Agent, Prefix, resume, Timeout).
+
+-spec suspend(Agent :: agent(),
+              Prefix :: service_name()) ->
+    module_response(ok | {error, any()}).
+
+suspend(Agent, Prefix) ->
+    cloudi:send_sync(Agent, Prefix, suspend).
+
+-spec suspend(Agent :: agent(),
+              Prefix :: service_name(),
+              Timeout :: timeout_milliseconds()) ->
+    module_response(ok | {error, any()}).
+
+suspend(Agent, Prefix, Timeout) ->
+    cloudi:send_sync(Agent, Prefix, suspend, Timeout).
+
 %%%------------------------------------------------------------------------
 %%% Callback functions from cloudi_service
 %%%------------------------------------------------------------------------
 
-cloudi_service_init(Args, Prefix, Timeout, _Dispatcher) ->
+cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
     Defaults = [
         {map_reduce,             ?DEFAULT_MAP_REDUCE_MODULE},
         {map_reduce_args,        ?DEFAULT_MAP_REDUCE_ARGUMENTS},
+        {name,                   ?DEFAULT_NAME},
         {concurrency,            ?DEFAULT_CONCURRENCY},
         {log_execution_time,     ?DEFAULT_LOG_EXECUTION_TIME}],
-    [MapReduceModule, MapReduceArgs, Concurrency, LogExecutionTime] =
+    [MapReduceModule, MapReduceArgs, Name, Concurrency, LogExecutionTime] =
         cloudi_proplists:take_values(Defaults, Args),
     TimeStart = cloudi_timestamp:seconds_monotonic(),
     true = is_atom(MapReduceModule) andalso (MapReduceModule /= undefined),
@@ -166,17 +215,20 @@ cloudi_service_init(Args, Prefix, Timeout, _Dispatcher) ->
     Service !  #init_begin{service = Service,
                            prefix = Prefix,
                            timeout = Timeout,
-                           time_start = TimeStart,
-                           log_execution_time = LogExecutionTime,
                            map_reduce_module = MapReduceModule,
                            map_reduce_args = MapReduceArgs,
-                           concurrency = Concurrency},
+                           map_reduce_name = Name,
+                           concurrency = Concurrency,
+                           log_execution_time = LogExecutionTime,
+                           time_start = TimeStart},
+    cloudi_service:subscribe(Dispatcher, Name),
     {ok, undefined}.
 
-cloudi_service_handle_request(_Type, _Name, _Pattern, _RequestInfo, _Request,
+cloudi_service_handle_request(_RequestType, _Name, _Pattern,
+                              _RequestInfo, Request,
                               _Timeout, _Priority, _TransId, _Pid,
-                              State, _Dispatcher) ->
-    {reply, <<>>, State}.
+                              State, Dispatcher) ->
+    request(Request, State, Dispatcher).
 
 cloudi_service_handle_info(#init_begin{service = Service} = InitBegin,
                            undefined, Dispatcher) ->
@@ -190,15 +242,16 @@ cloudi_service_handle_info(#init_begin{service = Service} = InitBegin,
     _ = erlang:spawn_link(fun() ->
         case init(InitBegin, Dispatcher) of
             {noreply, State} ->
-                Service ! #init_end{state = State};
+                Service ! #init_end{state = State},
+                ok;
             {stop, Reason, State} when Reason /= undefined ->
                 Service ! #init_end{state = State,
-                                    error = Reason}
+                                    error = Reason},
+                ok
         end,
         true = erlang:unlink(Service)
     end),
     {noreply, undefined};
-
 cloudi_service_handle_info(#init_end{state = State,
                                      error = Error},
                            undefined, _Dispatcher) ->
@@ -208,13 +261,11 @@ cloudi_service_handle_info(#init_end{state = State,
         true ->
             {stop, Error, State}
     end;
-
 cloudi_service_handle_info(Request, undefined, Dispatcher) ->
     % waiting for #init_end{} still
     erlang:send_after(?INFO_RETRY_INTERVAL,
                       cloudi_service:self(Dispatcher), Request),
     {noreply, undefined};
-
 cloudi_service_handle_info(#timeout_async_active{trans_id = TransId} = Request,
                            #state{map_reduce_module = MapReduceModule,
                                   map_reduce_state = MapReduceState,
@@ -222,19 +273,16 @@ cloudi_service_handle_info(#timeout_async_active{trans_id = TransId} = Request,
                            Dispatcher) ->
     case maps:find(TransId, MapRequests) of
         {ok, [_ | SendArgs]} ->
-            NextMapRequests = maps:remove(TransId, MapRequests),
-            case MapReduceModule:cloudi_service_map_reduce_resend(
-                [Dispatcher | SendArgs], MapReduceState) of
-                {ok, NewSendArgs, NewMapReduceState} ->
-                    case erlang:apply(cloudi_service, send_async_active,
-                                      NewSendArgs) of
-                        {ok, NewTransId} ->
-                            NewMapRequests = maps:put(NewTransId,
-                                                      NewSendArgs,
-                                                      NextMapRequests),
+            case MapReduceModule:
+                 cloudi_service_map_reduce_resend([Dispatcher | SendArgs],
+                                                  MapReduceState) of
+                {ok, SendArgsNew, MapReduceStateNew} ->
+                    case map_send_request(SendArgsNew,
+                                          maps:remove(TransId, MapRequests)) of
+                        {ok, MapRequestsNew} ->
                             {noreply,
-                             State#state{map_reduce_state = NewMapReduceState,
-                                         map_requests = NewMapRequests}};
+                             State#state{map_reduce_state = MapReduceStateNew,
+                                         map_requests = MapRequestsNew}};
                         {error, _} = Error ->
                             {stop, Error, State}
                     end;
@@ -244,7 +292,6 @@ cloudi_service_handle_info(#timeout_async_active{trans_id = TransId} = Request,
         error ->
             cloudi_service_map_reduce_info(Request, State, Dispatcher)
     end;
-
 cloudi_service_handle_info(#return_async_active{response_info = ResponseInfo,
                                                 response = Response,
                                                 timeout = Timeout,
@@ -255,49 +302,41 @@ cloudi_service_handle_info(#return_async_active{response_info = ResponseInfo,
                            Dispatcher) ->
     case maps:find(TransId, MapRequests) of
         {ok, [_ | SendArgs]} ->
-            case MapReduceModule:cloudi_service_map_reduce_recv(
-                [Dispatcher | SendArgs], ResponseInfo, Response,
-                Timeout, TransId, MapReduceState, Dispatcher) of
-                {ok, NextMapReduceState} ->
-                    case map_send(maps:remove(TransId, MapRequests),
-                                  Dispatcher, MapReduceModule,
-                                  NextMapReduceState) of
-                        {ok, NewMapRequests, NewMapReduceState} ->
-                            {noreply,
-                             State#state{map_reduce_state = NewMapReduceState,
-                                         map_requests = NewMapRequests}};
-                        {error, _} = Error ->
-                            {stop, Error, State}
-                    end;
-                {done, NewMapReduceState} ->
-                    NewMapRequests = maps:remove(TransId, MapRequests),
-                    NewState = State#state{map_reduce_state = NewMapReduceState,
-                                           map_requests = NewMapRequests},
-                    case maps:size(NewMapRequests) of
-                        0 ->
-                            {stop, shutdown, NewState};
-                        _ ->
-                            {noreply, NewState}
-                    end;
+            case MapReduceModule:
+                 cloudi_service_map_reduce_recv([Dispatcher | SendArgs],
+                                                ResponseInfo, Response,
+                                                Timeout, TransId,
+                                                MapReduceState, Dispatcher) of
+                {ok, MapReduceStateNew} ->
+                    MapRequestsNew = maps:remove(TransId, MapRequests),
+                    StateNew = State#state{map_reduce_state = MapReduceStateNew,
+                                           map_requests = MapRequestsNew},
+                    map_check_continue(StateNew, Dispatcher);
+                {done, MapReduceStateNew} ->
+                    MapRequestsNew = maps:remove(TransId, MapRequests),
+                    StateNew = State#state{map_reduce_state = MapReduceStateNew,
+                                           map_requests = MapRequestsNew},
+                    map_check_done(StateNew);
                 {error, _} = Error ->
                     {stop, Error, State}
             end;
         error ->
             cloudi_service_map_reduce_info(Request, State, Dispatcher)
     end;
-
 cloudi_service_handle_info(Request, State, Dispatcher) ->
     cloudi_service_map_reduce_info(Request, State, Dispatcher).
 
 cloudi_service_terminate(_Reason, _Timeout, undefined) ->
     ok;
 cloudi_service_terminate(shutdown, _Timeout,
-                         #state{time_start = TimeStart,
-                                log_execution_time = true,
-                                map_requests = #{}}) ->
-    TimeEnd = cloudi_timestamp:seconds_monotonic(),
+                         #state{log_execution_time = true,
+                                map_requests = #{},
+                                time_running = TimeRunningStart,
+                                elapsed_seconds = ElapsedSeconds}) ->
+    TimeRunningEnd = cloudi_timestamp:seconds_monotonic(),
+    ElapsedSecondsNew = ElapsedSeconds + (TimeRunningEnd - TimeRunningStart),
     ?LOG_INFO("total time taken was ~p hours",
-              [hours_elapsed(TimeEnd, TimeStart)]),
+              [hours_elapsed(ElapsedSecondsNew)]),
     ok;
 cloudi_service_terminate(_Reason, _Timeout, #state{}) ->
     ok.
@@ -308,28 +347,29 @@ cloudi_service_terminate(_Reason, _Timeout, #state{}) ->
 
 init(#init_begin{prefix = Prefix,
                  timeout = Timeout,
-                 time_start = TimeStart,
-                 log_execution_time = LogExecutionTime,
                  map_reduce_module = MapReduceModule,
                  map_reduce_args = MapReduceArgs,
-                 concurrency = Concurrency},
+                 map_reduce_name = MapReduceName,
+                 concurrency = Concurrency,
+                 log_execution_time = LogExecutionTime,
+                 time_start = TimeStart},
      Dispatcher) ->
     MapCount = cloudi_concurrency:count(Concurrency),
-    case MapReduceModule:cloudi_service_map_reduce_new(MapReduceArgs,
-                                                       MapCount,
-                                                       Prefix,
-                                                       Timeout,
-                                                       Dispatcher) of
+    case MapReduceModule:
+         cloudi_service_map_reduce_new(MapReduceArgs, MapCount,
+                                       Prefix, Timeout, Dispatcher) of
         {ok, MapReduceState} ->
-            case map_send(MapCount, #{}, Dispatcher,
-                          MapReduceModule, MapReduceState) of
-                {ok, MapRequests, NewMapReduceState} ->
-                    {noreply, #state{time_start = TimeStart,
-                                     log_execution_time = LogExecutionTime,
-                                     map_reduce_module = MapReduceModule,
-                                     map_reduce_state = NewMapReduceState,
-                                     map_count = MapCount,
-                                     map_requests = MapRequests}};
+            case map_send(MapCount, #{},
+                          MapReduceModule, MapReduceState, Dispatcher) of
+                {ok, MapRequests, MapReduceStateNew} ->
+                    {noreply,
+                     #state{map_reduce_module = MapReduceModule,
+                            map_reduce_state = MapReduceStateNew,
+                            map_reduce_name = MapReduceName,
+                            map_count = MapCount,
+                            log_execution_time = LogExecutionTime,
+                            map_requests = MapRequests,
+                            time_running = TimeStart}};
                 {error, _} = Error ->
                     {stop, Error, undefined}
             end;
@@ -337,53 +377,112 @@ init(#init_begin{prefix = Prefix,
             {stop, Error, undefined}
     end.
 
-map_send(MapRequests, Dispatcher, MapReduceModule, MapReduceState) ->
-    map_send(1, MapRequests, Dispatcher, MapReduceModule, MapReduceState).
-
-map_send(0, MapRequests, _Dispatcher, _MapReduceModule, MapReduceState) ->
-    {ok, MapRequests, MapReduceState};
-
-map_send(Count, MapRequests, Dispatcher, MapReduceModule, MapReduceState) ->
-    case MapReduceModule:cloudi_service_map_reduce_send(MapReduceState,
-                                                        Dispatcher) of
-        {ok, SendArgs, NewMapReduceState} ->
-            case erlang:apply(cloudi_service, send_async_active, SendArgs) of
-                {ok, TransId} ->
-                    map_send(Count - 1,
-                             maps:put(TransId, SendArgs, MapRequests),
-                             Dispatcher, MapReduceModule, NewMapReduceState);
-                {error, _} = Error ->
-                    Error
-            end;
-        {done, NewMapReduceState} ->
-            {ok, MapRequests, NewMapReduceState};
+request(_, undefined, _Dispatcher) ->
+    {reply, {error, init_pending}, undefined};
+request(suspend,
+        #state{map_reduce_module = MapReduceModule,
+               map_reduce_name = MapReduceName,
+               time_running = TimeRunningStart,
+               suspended = false,
+               elapsed_seconds = ElapsedSeconds} = State, _Dispatcher) ->
+    TimeRunningEnd = cloudi_timestamp:seconds_monotonic(),
+    ElapsedSecondsNew = ElapsedSeconds + (TimeRunningEnd - TimeRunningStart),
+    ?LOG_INFO("~s ~ts suspended", [MapReduceModule, MapReduceName]),
+    {reply, ok,
+     State#state{suspended = true,
+                 elapsed_seconds = ElapsedSecondsNew}};
+request(suspend,
+        #state{suspended = true} = State, _Dispatcher) ->
+    {reply, {error, already_suspended}, State};
+request(resume,
+        #state{map_reduce_module = MapReduceModule,
+               map_reduce_state = MapReduceState,
+               map_reduce_name = MapReduceName,
+               map_count = MapCount,
+               map_requests = MapRequests,
+               suspended = true} = State, Dispatcher) ->
+    TimeRunningStart = cloudi_timestamp:seconds_monotonic(),
+    case map_send(MapCount - maps:size(MapRequests), MapRequests,
+                  MapReduceModule, MapReduceState, Dispatcher) of
+        {ok, MapRequestsNew, MapReduceStateNew} ->
+            ?LOG_INFO("~s ~ts resumed", [MapReduceModule, MapReduceName]),
+            {reply, ok,
+             State#state{map_reduce_state = MapReduceStateNew,
+                         map_requests = MapRequestsNew,
+                         time_running = TimeRunningStart,
+                         suspended = false}};
         {error, _} = Error ->
-            Error
-    end.
+            {stop, Error, State}
+    end;
+request(resume,
+        #state{suspended = false} = State, _Dispatcher) ->
+    {reply, {error, already_resumed}, State}.
 
-cloudi_service_map_reduce_info(Request,
-                               #state{map_reduce_module = MapReduceModule,
-                                      map_reduce_state = MapReduceState,
-                                      map_requests = MapRequests} = State,
-                               Dispatcher) ->
-    case MapReduceModule:cloudi_service_map_reduce_info(Request,
-                                                        MapReduceState,
-                                                        Dispatcher) of
-        {ok, NewMapReduceState} ->
-            {noreply, State#state{map_reduce_state = NewMapReduceState}};
-        {done, NewMapReduceState} ->
-            NewState = State#state{map_reduce_state = NewMapReduceState},
-            case maps:size(MapRequests) of
-                0 ->
-                    {stop, shutdown, NewState};
-                _ ->
-                    {noreply, NewState}
-            end;
+cloudi_service_map_reduce_info(Request, State, Dispatcher) ->
+    #state{map_reduce_module = MapReduceModule,
+           map_reduce_state = MapReduceState} = State,
+    case MapReduceModule:
+         cloudi_service_map_reduce_info(Request, MapReduceState, Dispatcher) of
+        {ok, MapReduceStateNew} ->
+            {noreply, State#state{map_reduce_state = MapReduceStateNew}};
+        {done, MapReduceStateNew} ->
+            map_check_done(State#state{map_reduce_state = MapReduceStateNew});
         {error, _} = Error ->
             {stop, Error, State}
     end.
 
-hours_elapsed(Seconds1, Seconds0)
-    when Seconds1 >= Seconds0 ->
-    erlang:round(((Seconds1 - Seconds0) / (60 * 60)) * 10) / 10.
+map_send(MapRequests, MapReduceModule, MapReduceState, Dispatcher) ->
+    map_send(1, MapRequests, MapReduceModule, MapReduceState, Dispatcher).
+
+map_send(0, MapRequests, _MapReduceModule, MapReduceState, _Dispatcher) ->
+    {ok, MapRequests, MapReduceState};
+map_send(Count, MapRequests, MapReduceModule, MapReduceState, Dispatcher) ->
+    case MapReduceModule:
+         cloudi_service_map_reduce_send(MapReduceState, Dispatcher) of
+        {ok, SendArgs, MapReduceStateNew} ->
+            case map_send_request(SendArgs, MapRequests) of
+                {ok, MapRequestsNew} ->
+                    map_send(Count - 1, MapRequestsNew,
+                             MapReduceModule, MapReduceStateNew, Dispatcher);
+                {error, _} = Error ->
+                    Error
+            end;
+        {done, MapReduceStateNew} ->
+            {ok, MapRequests, MapReduceStateNew};
+        {error, _} = Error ->
+            Error
+    end.
+
+map_send_request(SendArgs, MapRequests) ->
+    case erlang:apply(cloudi_service, send_async_active, SendArgs) of
+        {ok, TransId} ->
+            {ok, maps:put(TransId, SendArgs, MapRequests)};
+        {error, _} = Error ->
+            Error
+    end.
+
+map_check_continue(#state{suspended = true} = State, _Dispatcher) ->
+    {noreply, State};
+map_check_continue(#state{map_reduce_module = MapReduceModule,
+                          map_reduce_state = MapReduceState,
+                          map_requests = MapRequests} = State, Dispatcher) ->
+    case map_send(MapRequests, MapReduceModule, MapReduceState, Dispatcher) of
+        {ok, MapRequestsNew, MapReduceStateNew} ->
+            {noreply,
+             State#state{map_reduce_state = MapReduceStateNew,
+                         map_requests = MapRequestsNew}};
+        {error, _} = Error ->
+            {stop, Error, State}
+    end.
+
+map_check_done(#state{map_requests = MapRequests} = State) ->
+    case maps:size(MapRequests) of
+        0 ->
+            {stop, shutdown, State};
+        _ ->
+            {noreply, State}
+    end.
+
+hours_elapsed(ElapsedSeconds) ->
+    erlang:round((ElapsedSeconds / (60 * 60)) * 10) / 10.
 
