@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2013-2019 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2013-2020 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -41,7 +41,6 @@ class _Task(threading.Thread):
         self.__api = api
 
     def run(self):
-        # pylint: disable=bare-except
         try:
             self.__check_environment()
             # idle service with no subscriptions
@@ -50,7 +49,7 @@ class _Task(threading.Thread):
             assert result is False
         except TerminateException:
             pass
-        except:
+        except Exception:
             traceback.print_exc(file=sys.stderr)
         print('terminate environment python_c')
 
@@ -71,7 +70,7 @@ class _Task(threading.Thread):
         assert os.environ['À_UNICODE'] == 'true'
         assert os.environ['UNICODE_À'] == 'true'
         assert os.environ['UNICODE_CHARACTER'] == 'À'
-        assert sys.argv[1:] == ['À','À','À','À']
+        assert sys.argv[1:] == ['', 'À','À','À','À']
 
 def _main():
     thread_count = API.thread_count()
