@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2011-2019 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -48,17 +48,20 @@ class Task(threading.Thread):
         """
         run the http_req thread
         """
-        # pylint: disable=bare-except
         try:
-            assert self.__api.subscribe_count(self.__name + '.xml/get') == 0
+            if self.__name == 'python':
+                assert self.__api.subscribe_count('python.xml/get') == 0
+
             self.__api.subscribe(self.__name + '.xml/get', self.__request)
-            assert self.__api.subscribe_count(self.__name + '.xml/get') == 1
+
+            if self.__name == 'python':
+                assert self.__api.subscribe_count('python.xml/get') == 1
 
             result = self.__api.poll()
             assert result is False
         except self.__terminate_exception:
             pass
-        except:
+        except Exception:
             traceback.print_exc(file=sys.stderr)
         print('terminate http_req %s' % self.__name)
 
@@ -66,7 +69,7 @@ class Task(threading.Thread):
                   timeout, priority, trans_id, pid):
         # pylint: disable=unused-argument
         # pylint: disable=too-many-arguments
-        http_qs = self.__api.info_key_value_parse(request)
+        http_qs = API.info_key_value_parse(request)
         value = http_qs.get(b'value', None)
         if value is None:
             response = """\
@@ -76,8 +79,11 @@ class Task(threading.Thread):
                 value = value[0]
             response = """\
 <http_test><value>%d</value></http_test>""" % (int(value),)
+        response_info = API.info_key_value_new({
+            b'content-type': b'text/xml; charset=utf-8',
+        })
         self.__api.return_(request_type, name, pattern,
-                           b'', response.encode('utf-8'),
+                           response_info, response.encode('utf-8'),
                            timeout, trans_id, pid)
 
 def _main():
