@@ -30,12 +30,12 @@
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
 %%% @copyright 2012-2020 Michael Truog
-%%% @version 1.8.1 {@date} {@time}
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_test_messaging).
 -author('mjtruog at protonmail dot com').
--vsn("1.8.1").
+-vsn("2.0.1").
 
 -behaviour(cloudi_service).
 
@@ -91,8 +91,9 @@ cloudi_service_init(_Args, Prefix, _Timeout, Dispatcher) ->
     case cloudi_service:process_index(Dispatcher) of
         0 ->
             ?LOG_TRACE("~p is sending", [?FUNCTION_NAME]),
-            cloudi_service:send_async(Dispatcher,
-                                      Prefix ++ "sequence1", "1");
+            {ok, _} = cloudi_service:send_async(Dispatcher,
+                                                Prefix ++ "sequence1", "1"),
+            ok;
         _ ->
             ok
     end,
@@ -115,8 +116,9 @@ cloudi_service_handle_request(_RequestType, _Name, Pattern,
             sequence1(Dispatcher, Prefix),
             ?LOG_INFO("messaging sequence1 end erlang"
                       " ~s (~s)", [Variation, Request]),
-            cloudi_service:send_async(Dispatcher,
-                                      Prefix ++ "sequence2", Request),
+            {ok, _} = cloudi_service:send_async(Dispatcher,
+                                                Prefix ++ "sequence2",
+                                                Request),
             {reply, "end", State};
         "a/b/c/d" ->
             true = Pattern == (Prefix ++ "a/b/c/d"),
@@ -168,8 +170,9 @@ cloudi_service_handle_request(_RequestType, _Name, Pattern,
             Sequence2RecvNew = sequence2(Dispatcher, Prefix, Sequence2Recv),
             ?LOG_INFO("messaging sequence2 end erlang"
                       " ~s (~s)", [Variation, Request]),
-            cloudi_service:send_async(Dispatcher,
-                                      Prefix ++ "sequence3", Request),
+            {ok, _} = cloudi_service:send_async(Dispatcher,
+                                                Prefix ++ "sequence3",
+                                                Request),
             {reply, "end", State#state{sequence2_recv = Sequence2RecvNew}};
         "e" ->
             if
@@ -197,9 +200,10 @@ cloudi_service_handle_request(_RequestType, _Name, Pattern,
             ?LOG_INFO("messaging sequence3 end erlang"
                       " ~s (~s)", [Variation, Request]),
             Iteration = erlang:list_to_integer(Request) + 1,
-            cloudi_service:send_async(Dispatcher,
-                                      Prefix ++ "sequence1",
-                                      erlang:integer_to_list(Iteration)),
+            RequestNew = erlang:integer_to_list(Iteration),
+            {ok, _} = cloudi_service:send_async(Dispatcher,
+                                                Prefix ++ "sequence1",
+                                                RequestNew),
             {reply, "end", State};
         "f1" ->
             RequestI = erlang:list_to_integer(Request),
@@ -262,63 +266,63 @@ sequence1(Dispatcher, Prefix) ->
                                                "azbzczd", <<"test15">>),
     % n.b., depends on cloudi_core_i_constants.hrl having
     % RECV_ASYNC_STRATEGY == recv_async_select_oldest
-    cloudi_service:recv_async(Dispatcher, Test1Id, false),
+    _ = cloudi_service:recv_async(Dispatcher, Test1Id, false),
     {ok, <<>>, Test1Response, Test1Id} =
         cloudi_service:recv_async(Dispatcher),
     <<"test1">> = Test1Response,
-    cloudi_service:recv_async(Dispatcher, Test2Id, false),
+    _ = cloudi_service:recv_async(Dispatcher, Test2Id, false),
     {ok, <<>>, Test2Response, Test2Id} =
         cloudi_service:recv_async(Dispatcher),
     <<"test2">> = Test2Response,
-    cloudi_service:recv_async(Dispatcher, Test3Id, false),
+    _ = cloudi_service:recv_async(Dispatcher, Test3Id, false),
     {ok, <<>>, Test3Response, Test3Id} =
         cloudi_service:recv_async(Dispatcher),
     <<"test3">> = Test3Response,
-    cloudi_service:recv_async(Dispatcher, Test4Id, false),
+    _ = cloudi_service:recv_async(Dispatcher, Test4Id, false),
     {ok, <<>>, Test4Response, Test4Id} =
         cloudi_service:recv_async(Dispatcher),
     <<"test4">> = Test4Response,
-    cloudi_service:recv_async(Dispatcher, Test5Id, false),
+    _ = cloudi_service:recv_async(Dispatcher, Test5Id, false),
     {ok, <<>>, Test5Response, Test5Id} =
         cloudi_service:recv_async(Dispatcher),
     <<"test5">> = Test5Response,
-    cloudi_service:recv_async(Dispatcher, Test6Id, false),
+    _ = cloudi_service:recv_async(Dispatcher, Test6Id, false),
     {ok, <<>>, Test6Response, Test6Id} =
         cloudi_service:recv_async(Dispatcher),
     <<"test6">> = Test6Response,
-    cloudi_service:recv_async(Dispatcher, Test7Id, false),
+    _ = cloudi_service:recv_async(Dispatcher, Test7Id, false),
     {ok, <<>>, Test7Response, Test7Id} =
         cloudi_service:recv_async(Dispatcher),
     <<"test7">> = Test7Response,
-    cloudi_service:recv_async(Dispatcher, Test8Id, false),
+    _ = cloudi_service:recv_async(Dispatcher, Test8Id, false),
     {ok, <<>>, Test8Response, Test8Id} =
         cloudi_service:recv_async(Dispatcher),
     <<"test8">> = Test8Response,
-    cloudi_service:recv_async(Dispatcher, Test9Id, false),
+    _ = cloudi_service:recv_async(Dispatcher, Test9Id, false),
     {ok, <<>>, Test9Response, Test9Id} =
         cloudi_service:recv_async(Dispatcher),
     <<"test9">> = Test9Response,
-    cloudi_service:recv_async(Dispatcher, Test10Id, false),
+    _ = cloudi_service:recv_async(Dispatcher, Test10Id, false),
     {ok, <<>>, Test10Response, Test10Id} =
         cloudi_service:recv_async(Dispatcher),
     <<"test10">> = Test10Response,
-    cloudi_service:recv_async(Dispatcher, Test11Id, false),
+    _ = cloudi_service:recv_async(Dispatcher, Test11Id, false),
     {ok, <<>>, Test11Response, Test11Id} =
         cloudi_service:recv_async(Dispatcher),
     <<"test11">> = Test11Response,
-    cloudi_service:recv_async(Dispatcher, Test12Id, false),
+    _ = cloudi_service:recv_async(Dispatcher, Test12Id, false),
     {ok, <<>>, Test12Response, Test12Id} =
         cloudi_service:recv_async(Dispatcher),
     <<"test12">> = Test12Response,
-    cloudi_service:recv_async(Dispatcher, Test13Id, false),
+    _ = cloudi_service:recv_async(Dispatcher, Test13Id, false),
     {ok, <<>>, Test13Response, Test13Id} =
         cloudi_service:recv_async(Dispatcher),
     <<"test13">> = Test13Response,
-    cloudi_service:recv_async(Dispatcher, Test14Id, false),
+    _ = cloudi_service:recv_async(Dispatcher, Test14Id, false),
     {ok, <<>>, Test14Response, Test14Id} =
         cloudi_service:recv_async(Dispatcher),
     <<"test14">> = Test14Response,
-    cloudi_service:recv_async(Dispatcher, Test15Id, false),
+    _ = cloudi_service:recv_async(Dispatcher, Test15Id, false),
     {ok, <<>>, Test15Response, Test15Id} =
         cloudi_service:recv_async(Dispatcher),
     <<"test15">> = Test15Response,
