@@ -3,7 +3,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2014-2017 Michael Truog <mjtruog at gmail dot com>
+# Copyright (c) 2014-2020 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -74,7 +74,7 @@ sub task
         }
         else
         {
-            print "$e";
+            print STDERR "$e";
         }
     }
     print "terminate msg_size perl\n";
@@ -83,12 +83,12 @@ sub task
 sub assert
 {
     my ($test) = @_;
-    CloudI::API->assert($test);
+    CloudI::API::assert($test);
 }
 
 {
     assert($use_threads);
-    my $thread_count = CloudI::API->thread_count();
+    my $thread_count = CloudI::API::thread_count();
     my @threads = ();
     for my $i (0 .. ($thread_count - 1))
     {
