@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2017-2018 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2017-2020 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -48,7 +48,6 @@ class Task(threading.Thread):
         """
         run the null thread
         """
-        # pylint: disable=bare-except
         try:
             self.__api.subscribe(self.__name + '/get', self.__request)
 
@@ -56,7 +55,7 @@ class Task(threading.Thread):
             assert result is False
         except self.__terminate_exception:
             pass
-        except:
+        except Exception:
             traceback.print_exc(file=sys.stderr)
         print('terminate null %s' % self.__name)
 
@@ -80,4 +79,3 @@ def _main():
 
 if __name__ == '__main__':
     _main()
-
